//*****************************************************************
//*    Procedura obslugi wyswietlacza graficznego LPH7583         *
//*                 na ukladzie PCF8558.                          *
//*    Tablica kodow ASCII umieszczona w pamieci EEPROM 24C02     *
//*    ver. 2.0                                                   *
//*    Autor: Andrzej Niedworok                                   *
//*****************************************************************
#include <LPH7583.H>

//Funkcja pobiera znak ASCII z pamieci EEPROM i umieszcza go w tablicy.
void LCD_Get_Char(unsigned char d){

 unsigned char i;

 for (i=0;i<6;i++){
  i2c_start();
  i2c_write(EEPROM_ADR + 0);              //Adres pamieci EEPROM + zapis
  i2c_write(d++);                         //Adres komorki pamieci EEPROM
  i2c_start();
  i2c_write(EEPROM_ADR + 1);              //Adres pamieci EEPROM + odczyt
  ascii_tab[i] = i2c_read(0);             //Odczytaj segment znaku ASCII
  i2c_stop();
 }
}

//Funcja wyswietla zadany tekst na LCD.
void LCDWrite(unsigned char data){

 unsigned char i;

 switch(data){

  case '0' : LCD_Get_Char(ADR_0);
   break;
  case '1' : LCD_Get_Char(ADR_1);
   break;
  case '2' : LCD_Get_Char(ADR_2);
   break;
  case '3' : LCD_Get_Char(ADR_3);
   break;
  case '4' : LCD_Get_Char(ADR_4);
   break;
  case '5' : LCD_Get_Char(ADR_5);
   break;
  case '6' : LCD_Get_Char(ADR_6);
   break;
  case '7' : LCD_Get_Char(ADR_7);
   break;
  case '8' : LCD_Get_Char(ADR_8);
   break;
  case '9' : LCD_Get_Char(ADR_9);
   break;
  case '.' : LCD_Get_Char(ADR_KROPKA);
   break;
  case ' ' : LCD_Get_Char(ADR_SPACJA);
   break;
  default :{                              //Gdy inny znak to...
   i2c_write(data);                       //Wyswiel jak leci
   return;                                //Wyjdz z funkcji
  }
 }
 i2c_start();
 i2c_write(0x7A);				               //Wyslij adres wyswietlacza
 i2c_write(y_pos);	                     //Ustaw kursor na zadanym wierszu
 i2c_write(x_pos);   		               //Ustaw kursor na zadanej kolumnie
 for(i=0;i<6;i++){
  i2c_write(ascii_tab[i]);                //Wyswietl znak umieszczony w tablicy
  x_pos++;                                //Zwieksz adres kolumny
 }
}

//Funkcja ustawia kursor na zadanej pozycji.
void LCDGoto(unsigned char x, unsigned char y){

 switch(y){
  case 0 : y_pos = 0x60;	   		      //Ustaw adres pierszego wiersza
   break;
  case 1 : y_pos = 0x61;			         //Ustaw adres drugiego wiersza
   break;
  case 2 : y_pos = 0x62;	         		//Ustaw adres trzeciego wiersza
   break;
  case 3 : y_pos = 0x63;         			//Ustaw adres czwartego wiersza
   break;
  case 4 : y_pos = 0x64;	         		//Ustaw adres piatego wiersza
   break;
 }
 x_pos = x;
 i2c_start();                             //Gdy wyswietlany znak spoza EEPROM to...
 i2c_write(LCD_ADR);                      
 i2c_write(y_pos);                        //Ustaw kursor na zadana kolumna
 i2c_write(x_pos);                        //Ustaw kursor na zadany wiersz
}

//Funkcja czysci ekran.
void LCDClear(void){

 unsigned char i, j;

 i2c_start();                             //Wyslij adres wyswietlacza
 i2c_write(LCD_ADR);	   	               //Ustaw kursor na pierwszej linii
 i2c_write(0x60);				               //Ustaw kursor na pierszej kolumnie
 i2c_write(0x00);
 for (i=0;i<6;i++)
  for (j=0;j<97;j++)
   i2c_write(0x00);			               //Wstaw pusty znak
 i2c_stop();
 i2c_start();
 i2c_write(LCD_ADR); 		               //Wyslij adres wyswietlacza
 i2c_write(0x60);				               //Ustaw kursor na pierwszej linii
 i2c_write(0x00);				               //Ustaw kursor na pierszej kolumnie
 i2c_stop();
}

//Funkcja wyswietla wskaznik postepu (bargraf).
void LCDBargraf(char x, char y, char dlugosc, char postemp){

 unsigned char i;

 LCDGoto(x, y);					            //Ustaw kursor na zadanej pozycji
 LCDWrite(0b01111111);				         //Wyswietl poczatek paska postempu
 for (i=0;i<dlugosc;i++){
  if (i < postemp)
   LCDWrite(0b01011101);	    	         //Wyswietl pelny pasek
  else
   LCDWrite(0b01000001);  			         //Wyswietl pusty pasek
 }
 LCDWrite(0b01111111);				         //Wyswietl koniec paska postempu
}
