;program Pomiar_R - pomiar rezystancji przez pomiar czasu ladowania kondensatora do 1/2 Vcc
;po nacisnieciu przycisku wykonuje pomiar Tm , Tx i przesyla dane do komputera za pomoca lacza szeregowego
;szybkosc transmisji 9600 bitow/sek
;format danych: Tm - MSB, LSB Tx - MSB, LSB

;podlaczenia:
;	GP0 - linia Rm
;	GP1 - linia R0
;	GP2 - linia Rx
;	GP3 - przycisk zerowania
;	GP4 - linia out (wyjscie TX RS232)
;	GP5 - przycisk klaw (pomiar)


#define out 	gpio, 4		;deklaracja linii out (GP4)		
#define	klaw	gpio, 5		;deklaracja linii klaw (GP5)


#define baud	.9600		;deklaracja szybkosci transmisji w bitach/sek
	

;==============================================================================
;wartosci stalych ntb i ptb dla roznych szybkosci transmisji 
;najblizsza wartosc dla Fclk = 4.000 MHz
;uwaga - nalezy skalibrowac wewnetrzny oscylator
;ntb = (((fclk/4)/baud)-13)/3, fclk w Hz, baud w bitach/sek

	;TABELA ntb i ptb (wartosci dziesietne)

	; 2400 bit/sek - ntb = .134 lub .135	ptb = .66
	; 4800 bit/sek - ntb = .65 				ptb = .31
	; 9600 bit/sek - ntb = .30 lub .31		ptb = .14
	;19200 bit/sek - ntb = .13				ptb =  .5
	;38400 bit/sek - ntbt = .4				ptb =  .1


	if	baud == .2400		;szybkosc 2400 bit/sek
	#define	ntb	.134		;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .4800		;szybkosc 4800 bit/sek
	#define	ntb	.65			;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .9600		;szybkosc 9600 bit/sek
	#define	ntb	.30			;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .19200		;szybkosc 19200 bit/sek
	#define	ntb	.13			;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .38400		;szybkosc 38400 bit/sek
	#define	ntb	.4			;stala do odmierzania czasu trwania 1 bitu 
	endif

	ifndef	ntb
	error	"Zla szybkosc transmisji"
	endif


;==============================================================================
	cblock	h'20'			;deklaracje adresow rejestrow

	tx_reg					;rejestr szeregowy nadajnika rs
	tx_cnt					;licznik bitow nadajnika rs
	del_reg					;licznik opoznienia nadajnika rs

	nr_kan					;numer kanalu
	ad_h, ad_l				;rejestry na wynik MSB i LSB

	endc
 
 	;deklaracja typu procesora i formatu liczb

	list p=12F675			;okreslenie typu procesora
	#include p12f675.inc	;dolaczenie pliku z nazwami

	;konfiguracja procesora, fclk = 4 MHz
	__config _INTRC_OSC_NOCLKOUT &_MCLRE_ON &_WDT_OFF

	errorlevel -302			;nie wyprowadzaj komunikatu nr 302
	
	org	h'00'				;adres poczatku programu
	goto	start			;skok do poczatku programu


;==============================================================================	
start		;pocztek programu

	;konfiguracja procesora

	;wylacz A/C
	bsf		status, rp0		;bank1
	clrf	ansel
	bcf		status, rp0		;bank0

	;konfiguruj komparator
	movlw	b'00010100'		;konfiguruj komparator z vref i wejsciem na gp1
	movwf	cmcon			;cinv=1

	movlw	b'00111101'		;ustal stan linii portu, GP0=1, GP1=0, GP2=1, GP4=1 
	movwf	gpio

	bsf		status, rp0		;bank 1

	;konfiguracja linii procesora
	movlw	b'00101101'		;GP1, GP4 - wyjscia, pozostale - wejscia
	movwf	trisio
	
	clrf	wpu				;zeruj wpu, po wyzerowaniu bity sa ustawione
	bsf		wpu, 5			;zalacz rez. podc. do GP5
	bcf		option_reg, not_gppu	;zalacz rezystory podciagajace


	;konfiguracja Vref
	movlw	b'10001000'		;konfiguruj vref =1/2 vdd, zalacz vref	
	movwf	vrcon

	bcf		status, rp0		;bank 0

	clrf	t1con			;konfiguruj timer 1, presk1=1, clk=tcm	


;==============================================================================
loop		;petla glowna 

test_klaw					;petla do oczekiwania na nacisniecie przycisku

	btfsc	klaw			;czy nacisniety przycisk
	goto	$-1				;nie, czekaj na nacisniecie przycisku	

	call	pomiar_tm		;zmierz czas tm

	call	wysw_tm			;wyswietla wynik pomiaru tm na terminalu

	call	pomiar_tx		;zmierz czas tx

	call	wysw_tx			;wyswietla wynik pomiaru tx na terminalu

	;organizacja wyswietlania wynikow:
	;Rm MSB LSB , Rx MSB LSB 

	movlw	h'0A'			;nowa linia
	call	tx_out

	movlw	h'0D'			;przejdz na poczatek linii
	call	tx_out

	;inne procedury w petli

	btfss	klaw			;czy puszczony przycisk
	goto	$-1				;nie, czekaj na puszczenie przycisku
							;i drgania stykow	


	goto 	loop			;idz do petli glownej


;==============================================================================
;procedury
;==============================================================================


;==============================================================================
pomiar_tm	;mierzy czas ladowania kondensatora przez Rm

	clrf	tmr1h			;zeruj tmr1h	
	clrf	tmr1l			;zeruj tmr1l

	movlw	b'00111101'		;ustal stan linii portu, GP0=1, GP1=0, GP2=1, GP4=1 
	movwf	gpio

	call	del_ntb			;poczekaj na rozladowanie

	bsf		status, rp0		;bank 1
	movlw	b'00101110'		;wylacz rozladowanie i zalacz ladowanie przez Rm
	movwf	trisio		
	bcf		status, rp0		;bank 0

	bsf		t1con, tmr1on	;zalacz tmr1

	btfss	cmcon, cout		;sprawdz wyjscie komparatora
	goto	$-1				;czekaj na 1 na wyjsciu komparatora

	bcf		t1con, tmr1on	;wylacz tmr1

	movlw	b'00111101'		;ustal stan linii portu, GP0=1, GP1=0, GP2=1, GP4=1 
	movwf	gpio

	bsf		status, rp0		;bank 1
	movlw	b'00101101'		;zalacz rozladowanie
	movwf	trisio
	bcf		status, rp0		;bank 0


	return


;==============================================================================
;Listing 7.4
;==============================================================================
pomiar_tx	;mierzy czas ladowania kondensatora przez Rx

	clrf	tmr1h			;zeruj tmr1h	
	clrf	tmr1l			;zeruj tmr1l

	movlw	b'00111101'		;ustal stan linii portu, GP0=1, GP1=0, GP2=1, GP4=1 
	movwf	gpio

	call	del_ntb			;poczekaj na rozladowanie

	bsf		status, rp0		;bank 1

	movlw	b'00101011'		;wylacz rozladowanie i zalacz ladowanie przez Rx
	movwf	trisio

	bcf		status, rp0		;bank 0

	bsf		t1con, tmr1on	;zalacz tmr1

	btfss	cmcon, cout		;sprawdz wyjscie komparatora
	goto	$-1				;czekaj na 1 na wyjsciu komparatora

	bcf		t1con,	tmr1on	;wylacz tmr1


	movlw	b'00111101'		;ustal stan linii portu, GP0=1, GP1=0, GP2=1, GP4=1 
	movwf	gpio

	bsf		status, rp0		;bank 1
	movlw	b'00101101'		;zalacz rozladowanie
	movwf	trisio
	bcf		status, rp0		;bank 0

	return


;==============================================================================
wysw_tm		;wyswietla wynik pomiaru tm

	movlw	A'T'			;litera T
	call	tx_out			;wyslij
	movlw	A'm'			;litera m
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij

	call	wysw			;wyswietl rm
	
	return


;==============================================================================
wysw_tx		;wyswietla wynik pomiaru tx

	movlw	A'T'			;litera T
	call	tx_out			;wyslij
	movlw	A'x'			;litera x
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij

	call	wysw			;wyswietl rx
	
	return


;==============================================================================
wysw	;wyswietla wyniki w kodzie ASCII 

		;przepisz wynik do lbyte i h byte
	movf	tmr1l, w		
	movwf	lbyte			;tmr1h -> lbyte
	movf	tmr1h, w		
	movwf	hbyte			;tmr1h -> hbyte

	call	bin2bcd			;zamien na bcd

;	;setki tys.
;	swapf	r2, w			;zamien polbajty r2 i wpisz do w
;	andlw	h'0F'			;tylko mlodszy bajt
;	addlw	.48				;zamiana na ASCII
;	call	tx_out			;wyslij

	;dzies. tys.
	movf	r2, w			;wpisz r2 do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij

	;tysiace
	swapf	r1, w			;zamien polbajty r1 i wpisz do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij
	;setki
	movf	r1, w			;wpisz r1 do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij
	;dziesiatki
	swapf	r0, w			;zamien polbajty r0 i wpisz do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij
	;jednosci
	movf	r0, w			;wpisz r0 do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij

	;dwie spacje odstepu
	movlw	.32				;kod spacji
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij


	return


;==============================================================================
tx_out		;wysyla bit startu, 8 bitow danych i bit stopu

	movwf	tx_reg			;wpisz dane do rejestru tx_reg

	movlw	.9
	movwf	tx_cnt			;licznik bitow (1+8)

	bcf	status, c			;wpisz bit startu

tx_loop		;petla do nadwania bitow
	
	btfss	status, c		;sprawdz c
	bcf		out				;c=0, zeruj out
	btfsc	status, c		;sprawdz c
	bsf		out				;c=1, ustaw out

	call	del_ntb			;opoznienie na czas 1 bitu
	
	rrf		tx_reg, f		;przesun do c kolejny bit

	decfsz	tx_cnt, f		;sprawdz czy nadano wszystkie bity
	goto	tx_loop			;nie, nadawaj kolejny bit

	bsf		status, c		;wpisz bit stopu

	btfss	status, c		;sprawdz c
	bcf		out				;c=0, zeruj out
	btfsc	status, c		;sprawdz c
	bsf		out				;c=1, ustaw out

	goto	$+1				;opoznienie wyrownawcze
	goto	$+1				;opoznienie wyrownawcze
	call	del_ntb			;opoznienie na czas 1 bitu, nadaje pierwszy bit stopu

;	drugi bit stopu
;	goto	$+1				;opoznienie wyrownawcze
;	goto	$+1				;opoznienie wyrownawcze
;	goto	$+1				;opoznienie wyrownawcze
;	call	del_ntb			;opoznienie na czas 1 bitu, nadaje drugi bit stopu

	return

del_ntb		;odmierza opoznienie na czas trwania 1 bitu

	movlw	ntb				;stala do odmierzania czasu 1 bitu	
	movwf	del_reg			;wpisz wartosc poczatkowa do licznika 

	decfsz	del_reg, f		;sprawdz czy del_reg =0 (petla do odliczania opoznienia)
	goto	$-1				;nie, czekaj dalej
	
	return


;==============================================================================
bin2bcd		;konwersja bin na bcd, wersja 16 bitowa

	;deklaracje rejestrow
	 cblock  
     	r0, r1, r2		    ;liczba dziesietna spakowana (6 cyfr)
     	lbyte, hbyte		;liczba binarna (2 bajty)
	 	cb0, cb1			;liczniki pomocnicze
     endc    

	;r0	dziesiatki 	jednosci
	;r1	tysiace		setki
	;r2	setki tys.	dzies. tys

	;lbyte	mlodszy bajt
	;hbyte	starszy bajt
	
		movlw   .16			;liczba bitow
        movwf   cb0			;wpisz do licznika bitow
        clrf    r0			;zeruj rejestry
        clrf    r1
        clrf    r2

bin2bcd1 
		movlw   r0
        movwf   fsr			;wskaznik na r0
		movlw	.3			;liczba bajtow wyniku
		movwf	cb1			;wpisz do licznika

bin2bcd2 		;korekta dziesietna
		movlw   h'33'		
        addwf   indf, f		;dodaj h'33'
        btfsc   indf, 3		;sprawdz czy dolny polbajt > 7
		andlw	h'F0'		;tak, zeruj dolny polbajt w
        btfsc   indf, 7		;sprawdz czy gorny polbajt > 7
		andlw	h'0F'		;tak, zeruj gorny polbajt w
        subwf   indf, f		;wynik =< 7, odejmij poprawke
        incf    fsr, f		;nastepny bajt danych
		decfsz	cb1			;nastepne bajty wyniku
        goto    bin2bcd2

		;przesun kolejny bit
		rlf     lbyte, f
        rlf     hbyte, f
        rlf     r0,f
        rlf     r1,f
        rlf     r2,f

        decfsz  cb0, f		;czy wszystkie bity
        goto    bin2bcd1

        retlw   0


;==============================================================================


	end
