;program klawisze - obsluga klawiatury do 16 przyciskow

;wykorzystuje pomiar rezystancji przez pomiar czasu ladowania kondensatora do 1/2 Vcc
;po zalaczeniu przycisku wykonuje pomiar Tm i oblicza dzielnik div
;po nacisnieciu przycisku wykonuje pomiar Tx i przesyla dane do komputera za pomoca lacza szeregowego
;szybkosc transmisji 9600 bitow/sek
;format danych: Tm - MSB, LSB Tx - MSB, LSB

;podlaczenia:
;	GP1 - linia pomiarowa
;	GP3 - przycisk zerowania
;	GP4 - linia out (wyjscie TX RS232)
;	GP5 - przycisk klaw (pomiar - odczyt nacisnietego klawisza)

#define	we		gpio, 1		;deklaracja linii we (GP1)
#define out 	gpio, 4		;deklaracja linii out (GP4)		
#define	klaw	gpio, 5		;deklaracja linii klaw (GP5)

#define	nklaw	.16+1		;liczba klawiszy +1

#define baud	.9600		;deklaracja szybkosci transmisji w bitach/sek
	

;==============================================================================
;wartosci stalych ntb i ptb dla roznych szybkosci transmisji 
;najblizsza wartosc dla Fclk = 4.000 MHz
;uwaga - nalezy skalibrowac wewnetrzny oscylator
;ntb = (((fclk/4)/baud)-13)/3, fclk w Hz, baud w bitach/sek

	;TABELA ntb i ptb (wartosci dziesietne)

	; 2400 bit/sek - ntb = .134 lub .135	ptb = .66
	; 4800 bit/sek - ntb = .65 				ptb = .31
	; 9600 bit/sek - ntb = .30 lub .31		ptb = .14
	;19200 bit/sek - ntb = .13				ptb =  .5
	;38400 bit/sek - ntbt = .4				ptb =  .1


	if	baud == .2400		;szybkosc 2400 bit/sek
	#define	ntb	.134		;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .4800		;szybkosc 4800 bit/sek
	#define	ntb	.65			;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .9600		;szybkosc 9600 bit/sek
	#define	ntb	.30			;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .19200		;szybkosc 19200 bit/sek
	#define	ntb	.13			;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .38400		;szybkosc 38400 bit/sek
	#define	ntb	.4			;stala do odmierzania czasu trwania 1 bitu 
	endif

	ifndef	ntb
	error	"Zla szybkosc transmisji"
	endif


;==============================================================================
	cblock	h'20'			;deklaracje adresow rejestrow

	tx_reg					;rejestr szeregowy nadajnika rs
	tx_cnt					;licznik bitow nadajnika rs
	del_reg					;licznik opoznienia nadajnika rs

	nr_kan					;numer kanalu
	ad_h, ad_l				;rejestry na wynik MSB i LSB

	div						;dzielnik
	xklaw					;nr klawisza

	endc
 
 	;deklaracja typu procesora i formatu liczb

	list p=12F675			;okreslenie typu procesora
	#include p12f675.inc	;dolaczenie pliku z nazwami

	;konfiguracja procesora, fclk = 4 MHz
	__config _INTRC_OSC_NOCLKOUT &_MCLRE_ON &_WDT_OFF

	errorlevel -302			;nie wyprowadzaj komunikatu nr 302
	
	org	h'00'				;adres poczatku programu
	goto	start			;skok do poczatku programu


;==============================================================================	
start		;pocztek programu

	;konfiguracja procesora

	;wylacz A/C
	bsf		status, rp0		;bank1
	clrf	ansel
	bcf		status, rp0		;bank0

	;konfiguruj komparator
	movlw	b'00010100'		;konfiguruj komparator z vref i wejsciem na gp1
	movwf	cmcon			;cinv=1

	movlw	b'00111111'		;ustal stan linii portu = 1
	movwf	gpio

	bsf	status, rp0			;bank 1

	;konfiguracja linii procesora
	movlw	b'00101000'		;GP0, GP1, GP2, GP4 - wyjscia, GP3 i GP5 - wejscia
	movwf	trisio
	
	clrf	wpu				;zeruj wpu, po wyzerowaniu bity sa ustawione
	bsf		wpu, 5			;zalacz rez. podc. do GP5
	bcf		option_reg, not_gppu	;zalacz rezystory podciagajace


	;konfiguracja Vref
	movlw	b'10001000'		;konfiguruj vref =1/2 vdd, zalacz vref	
	movwf	vrcon

	bcf		status, rp0		;bank 0

	clrf	t1con			;konfiguruj timer 1, presk1=1, clk=tcm	

	call 	pomiar_tm		;kalibracja - pomiar czasu Tm i obliczenie dzielnika div

	call	wysw_tm			;wyswietla tm i div na terminalu

	movlw	h'0A'			;nowa linia
	call	tx_out

	movlw	h'0D'			;przejdz na poczatek linii
	call	tx_out


;==============================================================================
loop		;petla glowna 

test_klaw					;petla do oczekiwania na nacisniecie przycisku
	btfsc	klaw			;czy nacisniety przycisk
	goto	test_klaw		;nie, czekaj na nacisniecie przycisku	

 	call	pomiar_tx		;zmierz czas tx i oblicz xklaw

	call	wysw_tx			;wyswietla tx i xklaw na terminalu


	movlw	h'0A'			;nowa linia
	call	tx_out

	movlw	h'0D'			;przejdz na poczatek linii
	call	tx_out

	;inne procedury w petli

	btfss	klaw			;czy puszczony przycisk
	goto	$-1				;nie, czekaj na puszczenie przycisku
							;i drgania stykow	


	goto 	loop			;idz do petli glownej


;==============================================================================
;procedury
;==============================================================================

;==============================================================================
pomiar_tm	;mierzy czas ladowania kondensatora przez wszystkie rezystory

	clrf	tmr1h			;zeruj tmr1h	
	clrf	tmr1l			;zeruj tmr1l

	call	rozladuj		;zalacz rozladowanie kondensatota
	call	del_ntb			;poczekaj na rozladowanie

	call	laduj			;zalacz ladowanie kondensatora
	bsf		t1con, tmr1on	;zalacz tmr1

	btfss	cmcon, cout		;sprawdz wyjscie komparatora
	goto	$-1				;czekaj na 1 na wyjsciu komparatora
	bcf		t1con, tmr1on	;wylacz tmr1

	;zapamietaj pomiar do wyswietlenia
	movf	tmr1l, w		
	movwf	lbyte			;wpisz tmr1l do lbyte
	movf	tmr1h, w
	movwf	hbyte			;wpisz tmr1h do hbyte

	;obliczenie dzielnika div


	clrf	div				;zeruj div
	movlw	nklaw			;wpisz nklaw do w

div_i		;obliczenie div (div = tmr1l/nklaw)
	
	incf	div, f			;inkrementuj div
	subwf	tmr1l, f		;odejmij nklaw
	btfsc	status, c		;czy roznica ujemna
	goto	div_i			;nie skok do div_i
	addwf	tmr1l, f
	movlw	((nklaw)/.2)
	subwf	tmr1l, f		;odejmij nklaw/2
	btfss	status, c		;czy oznica ujemna
	decf	div, f			;korekta div

	return


;==============================================================================
;Listing 7.5 Program identyfikacji nacisnietego klawisza
;==============================================================================
pomiar_tx	;mierzy czas ladowania kondensatora przez niezwarte rezystory

	clrf	tmr1h			;zeruj tmr1h	
	clrf	tmr1l			;zeruj tmr1l

	call	rozladuj		;zalacz rozladowanie kondensatota
	call	del_ntb			;poczekaj na rozladowanie

	call	laduj			;zalacz ladowanie kondensatora
	bsf		t1con, tmr1on	;zalacz tmr1


	btfss	cmcon, cout		;sprawdz wyjscie komparatora
	goto	$-1				;czekaj na 1 na wyjsciu komparatora
	bcf		t1con, tmr1on	;wylacz tmr1

	;zapamietaj pomiar do wyswietlenia
	movf	tmr1l, w		
	movwf	lbyte			;wpisz tmr1l do lbyte
	movf	tmr1h, w
	movwf	hbyte			;wpisz tmr1h do hbyte

	;obliczenie dzielnika xklaw
	
	bcf		status, c		;zeruj c
	rrf		div, w			;div/2 -> w
	addwf	tmr1l, f		;dodaj do tmr1l ze wzgledu na zaokraglenie

	clrf	xklaw			;zeruj xklaw
	movf	div, w			;wpisz div do w

xk_i		;obliczenie xklaw (xklaw = tmr1l/div)
	
	incf	xklaw, f		;inkrementuj xklaw
	subwf	tmr1l, f		;odejmij nklaw
	btfsc	status, c		;czy roznica ujemna
	goto	xk_i			;nie skok do xk_i
	addwf	tmr1l, f
	movlw	((nklaw)/.2)
	subwf	tmr1l, f		;odejmij nklaw/2
	btfss	status, c		;czy oznica ujemna
	decf	xklaw, f		;korekta xklaw

	return


;==============================================================================
rozladuj		;rozladowanie kondensatora
	bcf		we
	bsf		status, rp0		;bank 1
	bcf		we				;ustaw line we jako wyjscie i wpisz 0 
	bcf		status, rp0		;bank 0

	return


;==============================================================================
laduj		;ladowanie kondensatora

	bsf		status, rp0		;bank 1
	bsf		we				;ustaw linie we jako wejscie
	bcf		status, rp0		;bank 0

	return


;==============================================================================
wysw_tm		;wyswietla wynik pomiaru tm

	movlw	A'T'			;litera T
	call	tx_out			;wyslij
	movlw	A'm'			;litera m
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij

	call	wysw			;wyswietl tm

	movlw	A'd'			;litera d
	call	tx_out			;wyslij
	movlw	A'i'			;litera i
	call	tx_out			;wyslij
	movlw	A'v'			;litera v
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij

	;przepisz div do lbyte i hbyte
	clrf	hbyte			;zeruj hbyte
	movf	div, w		
	movwf	lbyte			;div -> lbyte
	call	wysw			;wyswietl div
	
	return


;==============================================================================
wysw_tx		;wyswietla wynik pomiaru tx

	movlw	A'T'			;litera T
	call	tx_out			;wyslij
	movlw	A'x'			;litera x
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij

	call	wysw			;wyswietl tm

	movlw	A'x'			;litera x
	call	tx_out			;wyslij
	movlw	A'k'			;litera k
	call	tx_out			;wyslij
	movlw	A'l'			;litera l
	call	tx_out			;wyslij
	movlw	A'a'			;litera a
	call	tx_out			;wyslij
	movlw	A'w'			;litera v
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij

	;przepisz div do lbyte i h byte
	clrf	hbyte			;zeruj hbyte
	movf	xklaw, w		
	movwf	lbyte			;xklaw -> lbyte
	
	;sprawdzamy czy xklaw =< nklaw
	movlw	nklaw
	subwf	xklaw, w		;xklaw - nklaw -> w
	skpnc					;czy wynik ujemny	
	clrf	lbyte			;nie, zeruj lbyte

	call	wysw			;wyswietl div

	return


;==============================================================================
wysw	;wyswietla wyniki w kodzie ASCII 

	call	bin2bcd			;zamien na bcd

;	;setki tys.
;	swapf	r2, w			;zamien polbajty r2 i wpisz do w
;	andlw	h'0F'			;tylko mlodszy bajt
;	addlw	.48				;zamiana na ASCII
;	call	tx_out			;wyslij

	;dzies. tys.
	movf	r2, w			;wpisz r2 do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij

	;tysiace
	swapf	r1, w			;zamien polbajty r1 i wpisz do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij
	;setki
	movf	r1, w			;wpisz r1 do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij
	;dziesiatki
	swapf	r0, w			;zamien polbajty r0 i wpisz do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij
	;jednosci
	movf	r0, w			;wpisz r0 do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij

	;dwie spacje odstepu
	movlw	.32				;kod spacji
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij

	return


;==============================================================================
tx_out		;wysyla bit startu, 8 bitow danych i bit stopu

	movwf	tx_reg			;wpisz dane do rejestru tx_reg

	movlw	.9
	movwf	tx_cnt			;licznik bitow (1+8)

	bcf	status, c			;wpisz bit startu

tx_loop		;petla do nadwania bitow
	
	btfss	status, c		;sprawdz c
	bcf		out				;c=0, zeruj out
	btfsc	status, c		;sprawdz c
	bsf		out				;c=1, ustaw out

	call	del_ntb			;opoznienie na czas 1 bitu
	
	rrf		tx_reg, f		;przesun do c kolejny bit

	decfsz	tx_cnt, f		;sprawdz czy nadano wszystkie bity
	goto	tx_loop			;nie, nadawaj kolejny bit

	bsf		status, c		;wpisz bit stopu

	btfss	status, c		;sprawdz c
	bcf		out				;c=0, zeruj out
	btfsc	status, c		;sprawdz c
	bsf		out				;c=1, ustaw out

	goto	$+1				;opoznienie wyrownawcze
	goto	$+1				;opoznienie wyrownawcze
	call	del_ntb			;opoznienie na czas 1 bitu, nadaje pierwszy bit stopu

;	drugi bit stopu
;	goto	$+1				;opoznienie wyrownawcze
;	goto	$+1				;opoznienie wyrownawcze
;	goto	$+1				;opoznienie wyrownawcze
;	call	del_ntb			;opoznienie na czas 1 bitu, nadaje drugi bit stopu

	return

del_ntb		;odmierza opoznienie na czas trwania 1 bitu

	movlw	ntb				;stala do odmierzania czasu 1 bitu	
	movwf	del_reg			;wpisz wartosc poczatkowa do licznika 

	decfsz	del_reg, f		;sprawdz czy del_reg =0 (petla do odliczania opoznienia)
	goto	$-1				;nie, czekaj dalej
	
	return


;==============================================================================
bin2bcd			;konwersja bin na bcd, wersja 16 bitowa

	;deklaracje rejestrow
	 cblock  
     	r0, r1, r2		    ;liczba dziesietna spakowana (6 cyfr)
     	lbyte, hbyte		;liczba binarna (2 bajty)
	 	cb0, cb1			;liczniki pomocnicze
     endc    

	;r0	dziesiatki 	jednosci
	;r1	tysiace		setki
	;r2	setki tys.	dzies. tys

	;lbyte	mlodszy bajt
	;hbyte	starszy bajt
	
 		movlw   .16			;liczba bitow
        movwf   cb0			;wpisz do licznika bitow
        clrf    r0
        clrf    r1
        clrf    r2

bin2bcd1 
		movlw   r0
        movwf   fsr			;wskaznik na r0
		movlw	.3			;liczba bajtow wyniku
		movwf	cb1			;wpisz do licznika

bin2bcd2 		;korekta dziesietna
		movlw   h'33'		
        addwf   indf, f		;dodaj h'33'
        btfsc   indf, 3		;sprawdz czy dolny polbajt > 7
		andlw	h'F0'		;tak, zeruj dolny polbajt w
        btfsc   indf, 7		;sprawdz czy gorny polbajt > 7
		andlw	h'0F'		;tak, zeruj gorny polbajt w
        subwf   indf, f		;wynik =< 7, odejmij poprawke
        incf    fsr, f		;nastepny bajt danych
		decfsz	cb1			;nastepne bajty wyniku
        goto    bin2bcd2

		;przesun kolejny bit
		rlf     lbyte, f
        rlf     hbyte, f
        rlf     r0,f
        rlf     r1,f
        rlf     r2,f

        decfsz  cb0, f		;czy wszystkie bity
        goto    bin2bcd1

        retlw   0


;==============================================================================


	end

