;adxl_out - program do odczytu przyspieszenia z akcelerometru
;wysyla co ok. 0.5 sek wartosci(w us): okresu pwm - t, czasu trwania impusow - tx i ty 
;wykorzystuje procedure read_adxl 

;zworki z2, z3, z4 - rozwarte
;zwarcie z4 - optyczna sygnalizacja transmisji rs

;	GP2	- linia out - (wyjscie TX RS232)
;	GP4 - linia wex	- pwmx
;	GP5	- linia wey - pwmy

#define out 	gpio, 2		;deklaracja linii out (GP2)	
	
#define we_x	gpio, 5		;deklaracja linii we_x (GP5)	
#define we_y	gpio, 4		;deklaracja linii we_y (GP4)	


;===============================================================
;wartosci stalych ntb i ptb dla roznych szybkosci transmisji 
;najblizsza wartosc dla Fclk = 4.000 MHz
;uwaga - nalezy skalibrowac wewnetrzny oscylator
;ntb = (((fclk/4)/baud)-13)/3, fclk w Hz, baud w bitach/sek

	;TABELA ntb i ptb (wartosci dziesietne)

	; 2400 bit/sek - ntb = .134 lub .135	ptb = .66
	; 4800 bit/sek - ntb = .65 				ptb = .31
	; 9600 bit/sek - ntb = .30 lub .31		ptb = .14
	;19200 bit/sek - ntb = .13				ptb =  .5
	;38400 bit/sek - ntbt = .4				ptb =  .1


#define	ntb	.134		;stala do odmierzania czasu trwania 1 bitu 

;===============================================================

	cblock	h'20'			;deklaracje adresow rejestrow

	tx_reg					;rejestr szeregowy nadajnika rs
	tx_cnt					;licznik bitow nadajnika rs
	del_reg					;licznik opoznienia nadajnika rs

	endc

;=============================================================== 

 	;deklaracja typu procesora i formatu liczb

	list p=12F675			;okreslenie typu procesora
	#include p12f675.inc	;dolaczenie pliku z nazwami

	;konfiguracja procesora, fclk = 4 MHz
	__config _INTRC_OSC_NOCLKOUT &_MCLRE_ON &_WDT_OFF

	errorlevel -302			;nie wyprowadzaj komunikatu nr 302
	
	org	h'00'				;adres poczatku programu

	goto	start			;skok do poczatku programu

	
start		;pocztek programu

	;konfiguracja procesora

	;wylacz komparator
	movlw	.7				;wylacz komparator
	movwf	cmcon			;cinv=1

	;wylacz A/C
	bsf		status, rp0		;bank1
	clrf	ansel

	;fabryczna kalibracja oscylatora
	call	h'3ff'			;fabryczna wartosc kalibracyjna -> w
	movwf	osccal			;w -> osccal
	bcf		status, rp0		;bank0

	movlw	b'00001100'		;inicjalizacja linii portu
	movwf	gpio

	bsf	status, rp0			;bank 1

	;konfiguracja linii procesora
	movlw	b'00111000'		;wejscia GP3,GP4,GP5,  wyjscia GP0,GP1,GP2
	movwf	trisio

	bcf		status, rp0		;bank 0


;================================================================================
loop		;petla glowna

	call	pom_adxl		;wykonaj pomiar czasow t, tx, ty

	call	wysw_pom_adxl	;wyswietl pomiary czasow t, tx, ty

	call	konfig_t1_05	;konfiguruj tmr1 do odliczania 0.5 sek

	btfss	pir1, t1if		;sprawdz flage t1if
	goto	$-1				;czekaj na przepelnienie timera 1
	bcf		pir1, t1if		;nastapilo przepelnienie timera 1 ( co 0.524 sek), zeruj flage przepelnienia
							
	goto	loop			;powrot do petli


;================================================================================

;================================================================================
;procedury
;================================================================================

;================================================================================
pom_adxl		;procedury do pomiaru czasow t, tx ty z adxl
				;wyniki w: t_h, t_l, tx_h, tx_l,ty_h, ty_l
				;zeruje licznik 1 (tmr1h i tmr1l)

;deklaracje rejestrow

	cblock

	t1_l, t1_h
	t2_l, t2_h
	t3_l, t3_h
	tx_l, tx_h
	ty_l, ty_h
	t_l, t_h

	endc

	clrf	t1con			; konfiguracja licznika 1, preskaler = 1, wewn. zegar

	bcf		intcon, gie		;wylacz przerwania
	bcf		t1con, tmr1on	;wylacz licznik 1

	clrf	tmr1l			;zeruj rejestry licznika 1
	clrf	tmr1h

	goto 	podstaw			;procedura testowa, wpisuje odczyty t1, t2, t3

;odczyt czasow t1, t2, t3 i obliczenia tx, ty i t

;==============================================================================
	;pomiar t1
ed_0	;czeka na 0 na we_x
	btfsc	we_x
	goto	ed_0			;czekaj na 0

ed_1	;czeka na zbocze + na we_x
	btfsc	we_x
	goto	ed_1			;czekaj na zbocze + na we_x

	bsf	t1con, tmr1on		;jest zbocze +, zalacz licznik 1
	
ed_2	;czeka na zbocze - na we_x
	btfsc	we_x
	goto	ed_2			;czekaj na zbocze - na we_x

	bcf	t1con, tmr1on		;jest zbocze -, wylacz licznik 1

	;odczytaj t1
	movf	tmr1l, w
	movwf	t1_l
	movf	tmr1h, w
	movwf	t1_h

	bsf	t1con, tmr1on		;zalacz licznik 1

;==============================================================================
	;pomiar t2
ed_3	;czeka na zbocze + na we_y
	btfsc	we_y
	goto	ed_3			;czekaj na zbocze + na we_y

	bcf	t1con, tmr1on		;jest zbocze -, wylacz licznik 1

	;odczytaj t2
	movf	tmr1l, w
	movwf	t2_l
	movf	tmr1h, w
	movwf	t2_h

	bsf		t1con, tmr1on	;zalacz licznik 1

;==============================================================================
	;pomiar t3
ed_4	;czeka na zbocze - na we_y
	btfsc	we_y
	goto	ed_4			;czekaj na zbocze - na we_y

	bcf	t1con, tmr1on		;jest zbocze -, wylacz licznik 1

	;odczytaj t3
	movf	tmr1l, w
	movwf	t3_l
	movf	tmr1h, w
	movwf	t3_h


podstaw				;procedura testowa, wpisuje odczyty t1, t2, t3

	movlw	.20
	movwf	t1_h
	movlw	.0
	movwf	t1_l

	movlw	.40
	movwf	t2_h
	movlw	.0
	movwf	t2_l

	movlw	.60
	movwf	t3_h
	movlw	.0
	movwf	t3_l


;==============================================================================
	;obliczenia tx, ty, t

	;t1 -> tx
	movf	t1_l, w
	movwf	tx_l
	movf	t1_h, w
	movwf	tx_h

;	;korekta t2 (o czas wylaczenia licznika 1)
;	movlw	.5
;	addwf	t2_l, f
;	skpnc
;	incf	t2_h, f
;
;	;korekta t3 (o czasy wylaczenia licznika 1)
;	movlw	.10
;	addwf	t3_l, f
;	skpnc
;	incf	t3_h, f

;==============================================================================
	;t3 -> ty
	movf	t3_l, w
	movwf	ty_l
	movf	t3_h, w
	movwf	ty_h

	;ty-t2 -> ty
	movf	t2_l, w
	subwf	ty_l, f
	skpc
	decf	ty_h, f	
	movf	t2_h, w
	subwf	ty_h, f
	
	;t3 + t2 -> t
	movf	t2_l, w
	addwf	t3_l, w
	movwf	t_l
	skpnc
	incf	t2_h, f
	movf	t2_h, w
	addwf	t3_h, w
	movwf	t_h

	;t  t1 -> t
	movf	t1_l, w
	subwf	t_l, f
	skpc
	decf	t_h, f	
	movf	t1_h, w
	subwf	t_h, f

	;t/2 ->t
	clrc
	rrf	t_h, f
	rrf	t_l, f

	bsf	intcon, gie			;zalacz przerwania

	return


;================================================================================
wysw_pom_adxl		;wyswietl pomiary czasow t, tx, ty
					;format: T pomiar t, Tx pomiar tx, Ty pomiar ty

	;wyswietl pomiar T
	movlw	A'T'			;kod litery T
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij

	;wpisz wynik do rejestrow hbyte i lbyte	
	movf	t_h, w			;wpisz t_h do w
	movwf	hbyte
	movf	t_l, w			;wpisz t_l do w
	movwf	lbyte

	call	wysw_wynik		;wyswietla wynik pomiaru 5 cyfr

	;dwie spacje odstepu
	movlw	.32				;kod spacji
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij

		;wyswietl pomiar Tx
	movlw	A'T'			;kod litery T
	call	tx_out			;wyslij
	movlw	A'x'			;kod litery x
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij

	;wpisz wynik do rejestrow hbyte i lbyte	
	movf	tx_h, w			;wpisz t_h do w
	movwf	hbyte
	movf	tx_l, w			;wpisz t_l do w
	movwf	lbyte

	call	wysw_wynik		;wyswietla wynik pomiaru 5 cyfr

	;dwie spacje odstepu
	movlw	.32				;kod spacji
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij


		;wyswietl pomiar Ty
	movlw	A'T'			;kod litery T
	call	tx_out			;wyslij
	movlw	A'y'			;kod litery x
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij

	;wpisz wynik do rejestrow hbyte i lbyte	
	movf	ty_h, w			;wpisz t_h do w
	movwf	hbyte
	movf	ty_l, w			;wpisz t_l do w
	movwf	lbyte

	call	wysw_wynik		;wyswietla wynik pomiaru 5 cyfr

	;dwie spacje odstepu
	movlw	.32				;kod spacji
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij


	movlw	h'0A'			;nowa linia
	call	tx_out

	movlw	h'0D'			;przejdz na poczatek linii
	call	tx_out

	return


;================================================================================
wysw_wynik		;wyswietla wynik pomiaru (5 cyfr)

	call	bin2bcd			;zamien na bcd

;	;setki tys.
;	swapf	r2, w			;zamien polbajty r2 i wpisz do w
;	andlw	h'0F'			;tylko mlodszy bajt
;	addlw	.48				;zamiana na ASCII
;	call	tx_out			;wyslij


	;dzies. tys.
	movf	r2, w			;wpisz r2 do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij

	;tysiace
	swapf	r1, w			;zamien polbajty r1 i wpisz do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij
	;setki
	movf	r1, w			;wpisz r1 do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij
	;dziesiatki
	swapf	r0, w			;zamien polbajty r0 i wpisz do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij
	;jednosci
	movf	r0, w			;wpisz r0 do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij

	;dwie spacje odstepu
	movlw	.32				;kod spacji
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij

	return


;================================================================================
tx_out		;wysyla bit startu, 8 bitow danych i bit stopu

	movwf	tx_reg			;wpisz dane do rejestru tx_reg

	movlw	.9
	movwf	tx_cnt				;licznik bitow (1+8)

	bcf	status, c			;wpisz bit startu

tx_loop		;petla do nadwania bitow
	
	btfss	status, c		;sprawdz c
	bcf		out				;c=0, zeruj out
	btfsc	status, c		;sprawdz c
	bsf		out				;c=1, ustaw out

	call	del_ntb			;opoznienie na czas 1 bitu
	
	rrf		tx_reg, f		;przesun do c kolejny bit

	decfsz	tx_cnt, f		;sprawdz czy nadano wszystkie bity
	goto	tx_loop			;nie, nadawaj kolejny bit

	bsf	status, c			;wpisz bit stopu

	btfss	status, c		;sprawdz c
	bcf		out				;c=0, zeruj out
	btfsc	status, c		;sprawdz c
	bsf		out				;c=1, ustaw out

	goto	$+1				;opoznienie wyrownawcze
	goto	$+1				;opoznienie wyrownawcze
	call	del_ntb			;opoznienie na czas 1 bitu, nadaje pierwszy bit stopu

;	drugi bit stopu
;	goto	$+1				;opoznienie wyrownawcze
;	goto	$+1				;opoznienie wyrownawcze
;	goto	$+1				;opoznienie wyrownawcze
;	call	del_ntb			;opoznienie na czas 1 bitu, nadaje drugi bit stopu

	return


;================================================================================
del_ntb		;odmierza opoznienie na czas trwania 1 bitu

	movlw	ntb				;stala do odmierzania czasu 1 bitu	
	movwf	del_reg			;wpisz wartosc poczatkowa do licznika 

	decfsz	del_reg, f		;sprawdz czy del_reg =0 (petla do odliczania opoznienia)
	goto	$-1				;nie, czekaj dalej	

	return


;================================================================================
;konwersja bin na bcd, wersja 16 bitowa

	;deklaracje rejestrow
	 cblock  
     	r0, r1, r2, r3	    ;liczba dziesietna spakowana (6 cyfr)
     	lbyte, hbyte		;liczba binarna (2 bajty)
	 	cb0, cb1			;liczniki pomocnicze
     endc    

	;r0	dziesiatki 	jednosci
	;r1	tysiace		setki
	;r2	setki tys.	dzies. tys

	;lbyte	mlodszy bajt
	;hbyte	starszy bajt
	
bin2bcd movlw   .16			;liczba bitow
        movwf   cb0			;wpisz do licznika bitow
        clrf    r0
        clrf    r1
        clrf    r2
        clrf    r3

bin2bcd1 
		movlw   r0
        movwf   fsr			;wskaznik na r0
		movlw	.3			;liczba bajtow wyniku
		movwf	cb1			;wpisz do licznika

bin2bcd2 		;korekta dziesietna
		movlw   h'33'		
        addwf   indf, f		;dodaj h'33'
        btfsc   indf, 3		;sprawdz czy dolny polbajt > 7
		andlw	h'F0'		;tak, zeruj dolny polbajt w
        btfsc   indf, 7		;sprawdz czy gorny polbajt > 7
		andlw	h'0F'		;tak, zeruj gorny polbajt w
        subwf   indf, f		;wynik =< 7, odejmij poprawke
        incf    fsr, f		;nastepny bajt danych
		decfsz	cb1			;nastepne bajty wyniku
        goto    bin2bcd2

		;przesun kolejny bit
		rlf     lbyte, f
        rlf     hbyte, f
        rlf     r0,f
        rlf     r1,f
        rlf     r2,f

        decfsz  cb0, f		;czy wszystkie bity
        goto    bin2bcd1

        retlw   0


;===============================================================================
konfig_t1_05			;konfiguracja licznika 1 do odliczania 0.5 sek


	movlw	b'00110000'		;preskaler = 8, wewn. zegar 
	movwf	t1con

	clrf	tmr1h			;zeruj rejestr tmr1h
	clrf	tmr1l			;zeruj rejestr tmr1l

	clrf	intcon			;zablokuj przerwania
	clrf	pir1			;zeruj flagi - t1if

	bsf		t1con, tmr1on	;zalacz timer 1

	return


;===============================================================================

	end
