;program del_sig - przetwornik a/c metoda delta-sigma
;po nacisnieciu przycisku wykonuje pomiar i przesyla dane do komputera za pomoca lacza szeregowego
;szybkosc transmisji 9600 bitow/sek
;zakres pomiarowy 0 - Vcc

;podlaczenia:

;	GP1 - linia pomiarowa - wejscie komparatora
;	GP2 - linia pomiarowa - ladowanie/rozladowanie
;	GP3 - przycisk zerowania
;	GP4 - linia out (wyjscie TX RS232)
;	GP5 - przycisk klaw (pomiar - odczyt nacisnietego klawisza)


#define out 	gpio, 4		;deklaracja linii out (GP4)		
#define	klaw	gpio, 5		;deklaracja linii klaw (GP5)

#define baud	.9600		;deklaracja szybkosci transmisji w bitach/sek
	

;==============================================================================
;wartosci stalych ntb i ptb dla roznych szybkosci transmisji 
;najblizsza wartosc dla Fclk = 4.000 MHz
;uwaga - nalezy skalibrowac wewnetrzny oscylator
;ntb = (((fclk/4)/baud)-13)/3, fclk w Hz, baud w bitach/sek

	;TABELA ntb i ptb (wartosci dziesietne)

	; 2400 bit/sek - ntb = .134 lub .135	ptb = .66
	; 4800 bit/sek - ntb = .65 				ptb = .31
	; 9600 bit/sek - ntb = .30 lub .31		ptb = .14
	;19200 bit/sek - ntb = .13				ptb =  .5
	;38400 bit/sek - ntbt = .4				ptb =  .1


	if	baud == .2400		;szybkosc 2400 bit/sek
	#define	ntb	.134		;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .4800		;szybkosc 4800 bit/sek
	#define	ntb	.65			;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .9600		;szybkosc 9600 bit/sek
	#define	ntb	.30			;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .19200		;szybkosc 19200 bit/sek
	#define	ntb	.13			;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .38400		;szybkosc 38400 bit/sek
	#define	ntb	.4			;stala do odmierzania czasu trwania 1 bitu 
	endif

	ifndef	ntb
	error	"Zla szybkosc transmisji"
	endif


;==============================================================================
	cblock	h'20'			;deklaracje adresow rejestrow

	tx_reg					;rejestr szeregowy nadajnika rs
	tx_cnt					;licznik bitow nadajnika rs
	del_reg					;licznik opoznienia nadajnika rs

	endc
 
 	;deklaracja typu procesora i formatu liczb

	list p=12F675			;okreslenie typu procesora
	#include p12f675.inc	;dolaczenie pliku z nazwami

	;konfiguracja procesora, fclk = 4 MHz
	__config _INTRC_OSC_NOCLKOUT &_MCLRE_ON &_WDT_OFF

	errorlevel -302			;nie wyprowadzaj komunikatu nr 302
	
	org	h'00'				;adres poczatku programu
	goto	start			;skok do poczatku programu


;==============================================================================	
start		;pocztek programu

	;konfiguracja procesora

	;wylacz A/C
	bsf		status, rp0		;bank1
	clrf	ansel
	bcf		status, rp0		;bank0


	movlw	b'00111111'		;ustal stan linii portu = 1
	movwf	gpio

	bsf	status, rp0			;bank 1

	;konfiguracja linii procesora
	movlw	b'00101010'		;GP0, GP2, GP4 - wyjscia, GP1, GP3 i GP5 - wejscia
	movwf	trisio
	
	clrf	wpu					;zeruj wpu, po wyzerowaniu bity sa ustawione
	bsf	wpu, 5					;zalacz rez. podc. do GP5
	bcf	option_reg, not_gppu	;zalacz rezystory podciagajace

	;konfiguracja Vref
	movlw	b'10001000'		;konfiguruj vref =1/2  vdd, zalacz vref	
	movwf	vrcon

	bcf		status, rp0		;bank 0

	;konfiguruj komparator
	movlw	b'00000011'		;konfiguruj komparator z vref i wejsciem na gp1 i wyjsciem na gp2
	movwf	cmcon			;cinv=0

	clrf	t1con			;konfiguruj timer 1, presk1=1, clk=tcm
	clrf	intcon			;zablokuj przerwania


;==============================================================================
loop		;petla glowna 

test_klaw					;petla do oczekiwania na nacisniecie przycisku

	btfsc	klaw			;czy nacisniety przycisk
	goto	test_klaw		;nie, czekaj na nacisniecie przycisku	

 	call	pom_del_sig		;wykonaj pomiar

	call	wysw			;wyswietla wynik na terminalu

	movlw	h'0A'			;nowa linia
	call	tx_out

	movlw	h'0D'			;przejdz na poczatek linii
	call	tx_out

	;inne procedury w petli

	btfss	klaw			;czy puszczony przycisk
	goto	$-1				;nie, czekaj na puszczenie przycisku

	call	del_20ms		;puszczony, czekaj 20 ms (drgania stykow)


	goto 	loop			;idz do petli glownej


;==============================================================================
;procedury
;==============================================================================


;==============================================================================
pom_del_sig		;pomiar napiecia metoda delta-sigma

	cblock
	result_l, result_h		;rejestry wyniku
	cnt_l, cnt_h			;liczniki cykli komp.
	endc
	
	;inicjalizacja zmiennych
	clrf cnt_l
	clrf cnt_h
	clrf result_l
	clrf result_h
	
	movlw 	0x04 			;konfig. komp. wewn. vref, gp1- wejscie
	movwf 	cmcon			;gp2  wyjscie cyfrowe
	

del_sig_loop		;petla glowna wykonywana 1024 razy

	btfsc 	cmcon, cout 	;sprawdz stan komparatora
	goto 	laduj			;vc < vref, cykl ladowania
				;vc > vref, cykl rozladowania
	
	nop 					;opoznienie 1 cykl
	bcf 	gpio, 2 		;gp2=0, rozladowanie
	incfsz 	result_l,f 		;inkrementuj, result_l, sprawdz przepelnienie
	goto	del_2 ;			;nie ma przepelnienia
	incf 	result_h,f 		;przepelnienia, inkrementuj, result_h,
	goto 	endloop 		;

laduj		;vc < vref, cykl ladowania

	bsf		gpio, 2 		;gp2=1, ladowanie
	nop 					;opoznienie 1 cykl
	goto 	del_2 			;opoznienie 2 cykle
	
del_2						;opoznienie 2 cykle
	goto	 endloop 		;opoznienie 2 cykle


endloop		;zliczanie cykli kompensacji

	incfsz 	cnt_l,f 		;inkrementuj cnt_l
	goto	del_5 			;nie ma przepelnienia
	incf 	cnt_h, f 		;przepelnienie, inkrementuj cnt_h
	movf 	cnt_h, w 		;cnt_h ->w
	andlw 	b'00000100' 	;maska na 3 bit(4x256=1024), rozdzielczosc 10 bitow

;	andlw 	b'00001000' 	;maska na 4 bit(8x256=2048), rozdzielczosc 11 bitow
;	andlw 	b'00010000' 	;maska na 5 bit(16x256=4096), rozdzielczosc 12 bitow
;	andlw 	b'00100000' 	;maska na 6 bit(32x256=8192), rozdzielczosc 13 bitow

	btfsc 	status,z 		;sprawdzenie czy wykonano 1024 cykle kompensacji
	goto 	del_sig_loop	;nie, wykonaj kolejny cykl
	goto	exit			;tak, koniec pomiaru

del_5		;opoznienie 5 cykli
	goto 	$+1 			;2 cykle
	nop 					;1 cykl
	goto 	del_sig_loop 			;2 cykle

exit		;koniec pomiaru

	movlw 	0x03 			;konfig. komp. wewn. vref, gp1 - wejscie
	movwf 	cmcon			;gp2  wyjscie komparatora (pwm)

	return


;==============================================================================
wysw	;wyswietla wynik w kodzie ASCII 

	;przepisz wynik do hbyte i lbyte
	movf	result_h, w
	movwf	hbyte			;result_h -> hbyte
	movf	result_l, w
	movwf	lbyte			;result_l -> lbyte	

	call	bin2bcd			;zamien na bcd

;	;setki tys.
;	swapf	r2, w			;zamien polbajty r2 i wpisz do w
;	andlw	h'0F'			;tylko mlodszy bajt
;	addlw	.48				;zamiana na ASCII
;	call	tx_out			;wyslij

;	;dzies. tys.
;	movf	r2, w			;wpisz r2 do w
;	andlw	h'0F'			;tylko mlodszy bajt
;	addlw	.48				;zamiana na ASCII
;	call	tx_out			;wyslij
;
	;tysiace
	swapf	r1, w			;zamien polbajty r1 i wpisz do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij
	;setki
	movf	r1, w			;wpisz r1 do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij
	;dziesiatki
	swapf	r0, w			;zamien polbajty r0 i wpisz do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij
	;jednosci
	movf	r0, w			;wpisz r0 do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij

	;dwie spacje odstepu
	movlw	.32				;kod spacji
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij

	return


;==============================================================================
tx_out		;wysyla bit startu, 8 bitow danych i bit stopu

	movwf	tx_reg			;wpisz dane do rejestru tx_reg

	movlw	.9
	movwf	tx_cnt			;licznik bitow (1+8)

	bcf	status, c			;wpisz bit startu

tx_loop		;petla do nadwania bitow
	
	btfss	status, c		;sprawdz c
	bcf		out				;c=0, zeruj out
	btfsc	status, c		;sprawdz c
	bsf		out				;c=1, ustaw out

	call	del_ntb			;opoznienie na czas 1 bitu
	
	rrf		tx_reg, f		;przesun do c kolejny bit

	decfsz	tx_cnt, f		;sprawdz czy nadano wszystkie bity
	goto	tx_loop			;nie, nadawaj kolejny bit

	bsf		status, c		;wpisz bit stopu

	btfss	status, c		;sprawdz c
	bcf		out				;c=0, zeruj out
	btfsc	status, c		;sprawdz c
	bsf		out				;c=1, ustaw out

	goto	$+1				;opoznienie wyrownawcze
	goto	$+1				;opoznienie wyrownawcze
	call	del_ntb			;opoznienie na czas 1 bitu, nadaje pierwszy bit stopu

;	drugi bit stopu
;	goto	$+1				;opoznienie wyrownawcze
;	goto	$+1				;opoznienie wyrownawcze
;	goto	$+1				;opoznienie wyrownawcze
;	call	del_ntb			;opoznienie na czas 1 bitu, nadaje drugi bit stopu

	return

del_ntb		;odmierza opoznienie na czas trwania 1 bitu

	movlw	ntb				;stala do odmierzania czasu 1 bitu	
	movwf	del_reg			;wpisz wartosc poczatkowa do licznika 

	decfsz	del_reg, f		;sprawdz czy del_reg =0 (petla do odliczania opoznienia)
	goto	$-1				;nie, czekaj dalej
	
	return


;==============================================================================
bin2bcd 		;konwersja bin na bcd, wersja 16 bitowa

	;deklaracje rejestrow
	 cblock  
     	r0, r1, r2		    ;liczba dziesietna spakowana (6 cyfr)
     	lbyte, hbyte		;liczba binarna (2 bajty)
	 	cb0, cb1			;liczniki pomocnicze
     endc    

	;r0	dziesiatki 	jednosci
	;r1	tysiace		setki
	;r2	setki tys.	dzies. tys

	;lbyte	mlodszy bajt
	;hbyte	starszy bajt
	
		movlw   .16			;liczba bitow
        movwf   cb0			;wpisz do licznika bitow
        clrf    r0
        clrf    r1
        clrf    r2

bin2bcd1 
		movlw   r0
        movwf   fsr			;wskaznik na r0
		movlw	.3			;liczba bajtow wyniku
		movwf	cb1			;wpisz do licznika

bin2bcd2 		;korekta dziesietna
		movlw   h'33'		
        addwf   indf, f		;dodaj h'33'
        btfsc   indf, 3		;sprawdz czy dolny polbajt > 7
		andlw	h'F0'		;tak, zeruj dolny polbajt w
        btfsc   indf, 7		;sprawdz czy gorny polbajt > 7
		andlw	h'0F'		;tak, zeruj gorny polbajt w
        subwf   indf, f		;wynik =< 7, odejmij poprawke
        incf    fsr, f		;nastepny bajt danych
		decfsz	cb1			;nastepne bajty wyniku
        goto    bin2bcd2

		;przesun kolejny bit
		rlf     lbyte, f
        rlf     hbyte, f
        rlf     r0,f
        rlf     r1,f
        rlf     r2,f

        decfsz  cb0, f		;czy wszystkie bity
        goto    bin2bcd1

        retlw   0


;==============================================================================
del_20ms	;opoznienie  ok 20 ms na eliminacje drgan stykow
	
	movlw	.177
	movwf	tmr1h			;wpisz .177 do tmr1h	
	clrf	tmr1l			;zeruj tmr1l
	bcf		pir1, tmr1if	;zeruj flage tmr1if
	bsf		t1con, tmr1on	;zalacz tmr1

	btfss	pir1, tmr1if	;sprawdz flage tmr1if
	goto	$-1				;czekaj na ustwienie flagi 

	bcf		t1con, tmr1on	;wylacz tmr1
	bcf		pir1, tmr1if	;zeruj flage tmr1if

	return


;==============================================================================


	end




