;program n_klaw - obsluga klawiatury do 16 przyciskow

;wykorzystuje pomiar rezystancji przez pomiar czasu ladowania kondensatora do 1/2 Vcc
;po zalaczeniu przycisku wykonuje pomiar Tm i oblicza dzielnik div
;po nacisnieciu przycisku wykonuje pomiar Tx i przesyla dane do komputera za pomoca lacza szeregowego
;szybkosc transmisji 9600 bitow/sek
;format danych: Tm - MSB, LSB Tx - MSB, LSB

;podlaczenia:
;	GP1 - linia pomiarowa
;	GP3 - przycisk zerowania
;	GP4 - linia out (wyjscie TX RS232)
;	GP5 - przycisk klaw (pomiar - odczyt nacisnietego klawisza)

#define	we		gpio, 1		;deklaracja linii we (GP1)
#define out 	gpio, 4		;deklaracja linii out (GP4)		
#define	klaw	gpio, 5		;deklaracja linii klaw (GP5)

#define	nklaw	.16+1		;liczba klawiszy +1

#define baud	.9600		;deklaracja szybkosci transmisji w bitach/sek
	

;===============================================================
;wartosci stalych ntb i ptb dla roznych szybkosci transmisji 
;najblizsza wartosc dla Fclk = 4.000 MHz
;uwaga - nalezy skalibrowac wewnetrzny oscylator
;ntb = (((fclk/4)/baud)-13)/3, fclk w Hz, baud w bitach/sek

	;TABELA ntb i ptb (wartosci dziesietne)

	; 2400 bit/sek - ntb = .134 lub .135	ptb = .66
	; 4800 bit/sek - ntb = .65 				ptb = .31
	; 9600 bit/sek - ntb = .30 lub .31		ptb = .14
	;19200 bit/sek - ntb = .13				ptb =  .5
	;38400 bit/sek - ntbt = .4				ptb =  .1


	if	baud == .2400		;szybkosc 2400 bit/sek
	#define	ntb	.134		;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .4800		;szybkosc 4800 bit/sek
	#define	ntb	.65			;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .9600		;szybkosc 9600 bit/sek
	#define	ntb	.30			;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .19200		;szybkosc 19200 bit/sek
	#define	ntb	.13			;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .38400		;szybkosc 38400 bit/sek
	#define	ntb	.4			;stala do odmierzania czasu trwania 1 bitu 
	endif

	ifndef	ntb
	error	"Zla szybkosc transmisji"
	endif



	cblock	h'20'			;deklaracje adresow rejestrow

	tx_reg					;rejestr szeregowy nadajnika rs
	tx_cnt					;licznik bitow nadajnika rs
	del_reg					;licznik opoznienia nadajnika rs

	nr_kan					;numer kanalu
	ad_h, ad_l				;rejestry na wynik MSB i LSB

	div						;dzielnik
	xklaw					;nr klawisza

	endc
 
 	;deklaracja typu procesora i formatu liczb

	list p=12F675			;okreslenie typu procesora
	#include p12f675.inc	;dolaczenie pliku z nazwami

	;konfiguracja procesora, fclk = 4 MHz
	__config _INTRC_OSC_NOCLKOUT &_MCLRE_ON &_WDT_OFF

	errorlevel -302			;nie wyprowadzaj komunikatu nr 302
	
	org	h'00'				;adres poczatku programu
	goto	start			;skok do poczatku programu

	
start		;pocztek programu

	;konfiguracja procesora

	;wylacz A/C
	bsf		status, rp0		;bank1
	clrf	ansel
	bcf		status, rp0		;bank0

	;konfiguruj komparator
	movlw	b'00010100'		;konfiguruj komparator z vref i wejsciem na gp1
	movwf	cmcon			;cinv=1

	movlw	b'00111111'		;ustal stan linii portu = 1
	movwf	gpio

	bsf	status, rp0			;bank 1

	;konfiguracja linii procesora
	movlw	b'00101000'		;GP0, GP1, GP2, GP4 - wyjscia, GP3 i GP5 - wejscia
	movwf	trisio
	
	clrf	wpu					;zeruj wpu, po wyzerowaniu bity sa ustawione
	bsf	wpu, 5					;zalacz rez. podc. do GP5
	bcf	option_reg, not_gppu	;zalacz rezystory podciagajace


	;konfiguracja Vref
	movlw	b'10000111'		;konfiguruj vref =1/2-  vdd, zalacz vref	
	movwf	vrcon

	bcf		status, rp0		;bank 0

	clrf	t1con			;konfiguruj timer 1, presk1=1, clk=tcm	


	call 	pomiar_tm		;kalibracja - pomiar czasu Tm i obliczenie dzielnika div

	call	wysw_tm			;wyswietla tm i div na terminalu


	movlw	h'0A'			;nowa linia
	call	tx_out

	movlw	h'0D'			;przejdz na poczatek linii
	call	tx_out



loop		;petla glowna 

test_klaw					;petla do oczekiwania na nacisniecie przycisku

	btfsc	klaw			;czy nacisniety przycisk
	goto	$-1				;nie, czekaj na nacisniecie przycisku	

 	call	pomiar_tx		;zmierz czas tx i oblicz xklaw

	call	wysw_tx			;wyswietla tx i xklaw na terminalu


	movlw	h'0A'			;nowa linia
	call	tx_out

	movlw	h'0D'			;przejdz na poczatek linii
	call	tx_out

	;inne procedury w petli

	btfss	klaw			;czy puszczony przycisk
	goto	$-1				;nie, czekaj na puszczenie przycisku
							;i drgania stykow	


	goto 	loop			;idz do petli glownej

;================================================================================
;================================================================================
;procedury
;================================================================================
;================================================================================

;================================================================================
pomiar_tm	;mierzy czas ladowania kondensatora przez wszystkie rezystory

	clrf	tmr1h			;zeruj tmr1h	
	clrf	tmr1l			;zeruj tmr1l

	call	rozladuj		;zalacz rozladowanie kondensatota
	call	del_ntb			;poczekaj na rozladowanie

	call	laduj			;zalacz ladowanie kondensatora
	bsf		t1con, tmr1on	;zalacz tmr1

	btfss	cmcon, cout		;sprawdz wyjscie komparatora
	goto	$-1				;czekaj na 1 na wyjsciu komparatora
	bcf		t1con, tmr1on	;wylacz tmr1

	;zapamietaj pomiar do wyswietlenia
	movf	tmr1l, w		
	movwf	lbyte			;wpisz tmr1l do lbyte
	movf	tmr1h, w
	movwf	hbyte			;wpisz tmr1h do hbyte

	;obliczenie dzielnika div


	clrf	div				;zeruj div
	movlw	nklaw			;wpisz nklaw do w

div_i		;obliczenie div (div = tmr1l/nklaw)
	
	incf	div, f			;inkrementuj div
	subwf	tmr1l, f		;odejmij nklaw
	btfsc	status, c		;czy roznica ujemna
	goto	div_i			;nie skok do div_i
	addwf	tmr1l, f
	movlw	((nklaw)/.2)
	subwf	tmr1l, f		;odejmij nklaw/2
	btfss	status, c		;czu oznica ujemna
	decf	div, f			;korekta div

	return


;================================================================================
pomiar_tx	;mierzy czas ladowania kondensatora przez niezwarte rezystory

	clrf	tmr1h			;zeruj tmr1h	
	clrf	tmr1l			;zeruj tmr1l

	call	rozladuj		;zalacz rozladowanie kondensatota
	call	del_ntb			;poczekaj na rozladowanie

	call	laduj			;zalacz ladowanie kondensatora
	bsf		t1con, tmr1on	;zalacz tmr1


	btfss	cmcon, cout		;sprawdz wyjscie komparatora
	goto	$-1				;czekaj na 1 na wyjsciu komparatora
	bcf		t1con, tmr1on	;wylacz tmr1

	;zapamietaj pomiar do wyswietlenia
	movf	tmr1l, w		
	movwf	lbyte			;wpisz tmr1l do lbyte
	movf	tmr1h, w
	movwf	hbyte			;wpisz tmr1h do hbyte

	;obliczenie dzielnika xklaw
	
	bcf		status, c		;zeruj c
	rrf		div, w			;div/2 -> w
	addwf	tmr1l, f		;dodaj do tmr1l ze wzgledu na zaokraglenie

	clrf	xklaw			;zeruj xklaw
	movf	div, w			;wpisz div do w

xk_i		;obliczenie xklaw (xklaw = tmr1l/div)
	
	incf	xklaw, f		;inkrementuj xklaw
	subwf	tmr1l, f		;odejmij nklaw
	btfsc	status, c		;czy roznica ujemna
	goto	xk_i			;nie skok do xk_i
	addwf	tmr1l, f
	movlw	((nklaw)/.2)
	subwf	tmr1l, f		;odejmij nklaw/2
	btfss	status, c		;czy oznica ujemna
	decf	xklaw, f		;korekta xklaw

	return


;================================================================================
rozladuj		;rozladowanie kondensatora
	bcf		we
	bsf		status, rp0		;bank 1
	bcf		we				;ustaw line we jako wyjscie i wpisz 0 
	bcf		status, rp0		;bank 0

	return


;================================================================================
laduj		;ladowanie kondensatora

	bsf		status, rp0		;bank 1
	bsf		we				;ustaw linie we jako wejscie
	bcf		status, rp0		;bank 0

	return


;================================================================================
wysw_tm		;wyswietla wynik pomiaru tm

	movlw	A'T'			;litera T
	call	tx_out			;wyslij
	movlw	A'm'			;litera m
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij

	call	wysw			;wyswietl tm

	movlw	A'd'			;litera d
	call	tx_out			;wyslij
	movlw	A'i'			;litera i
	call	tx_out			;wyslij
	movlw	A'v'			;litera v
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij

	;przepisz div do lbyte i hbyte
	clrf	hbyte			;zeruj hbyte
	movf	div, w		
	movwf	lbyte			;div -> lbyte
	call	wysw			;wyswietl div
	
	return


;================================================================================
wysw_tx		;wyswietla wynik pomiaru tx

	movlw	A'T'			;litera T
	call	tx_out			;wyslij
	movlw	A'x'			;litera x
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij

	call	wysw			;wyswietl tm

	movlw	A'x'			;litera x
	call	tx_out			;wyslij
	movlw	A'k'			;litera k
	call	tx_out			;wyslij
	movlw	A'l'			;litera l
	call	tx_out			;wyslij
	movlw	A'a'			;litera a
	call	tx_out			;wyslij
	movlw	A'w'			;litera v
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij

	;przepisz div do lbyte i h byte
	clrf	hbyte			;zeruj hbyte
	movf	xklaw, w		
	movwf	lbyte			;xklaw -> lbyte
	
	;sprawdzamy czy xklaw =< nklaw
	movlw	nklaw
	subwf	xklaw, w		;xklaw - nklaw -> w
	skpnc					;czy wynik ujemny	
	clrf	lbyte			;nie, zeruj lbyte

	call	wysw			;wyswietl div

	return


;================================================================================
wysw	;wyswietla wyniki w kodzie ASCII 

	call	bin2bcd			;zamien na bcd

;	;setki tys.
;	swapf	r2, w			;zamien polbajty r2 i wpisz do w
;	andlw	h'0F'			;tylko mlodszy bajt
;	addlw	.48				;zamiana na ASCII
;	call	tx_out			;wyslij

	;dzies. tys.
	movf	r2, w			;wpisz r2 do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij

	;tysiace
	swapf	r1, w			;zamien polbajty r1 i wpisz do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij
	;setki
	movf	r1, w			;wpisz r1 do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij
	;dziesiatki
	swapf	r0, w			;zamien polbajty r0 i wpisz do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij
	;jednosci
	movf	r0, w			;wpisz r0 do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij

	;dwie spacje odstepu
	movlw	.32				;kod spacji
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij

	return


;================================================================================
tx_out		;wysyla bit startu, 8 bitow danych i bit stopu

	movwf	tx_reg			;wpisz dane do rejestru tx_reg

	movlw	.9
	movwf	tx_cnt			;licznik bitow (1+8)

	bcf	status, c			;wpisz bit startu

tx_loop		;petla do nadwania bitow
	
	btfss	status, c		;sprawdz c
	bcf		out				;c=0, zeruj out
	btfsc	status, c		;sprawdz c
	bsf		out				;c=1, ustaw out

	call	del_ntb			;opoznienie na czas 1 bitu
	
	rrf		tx_reg, f		;przesun do c kolejny bit

	decfsz	tx_cnt, f		;sprawdz czy nadano wszystkie bity
	goto	tx_loop			;nie, nadawaj kolejny bit

	bsf		status, c		;wpisz bit stopu

	btfss	status, c		;sprawdz c
	bcf		out				;c=0, zeruj out
	btfsc	status, c		;sprawdz c
	bsf		out				;c=1, ustaw out

	goto	$+1				;opoznienie wyrownawcze
	goto	$+1				;opoznienie wyrownawcze
	call	del_ntb			;opoznienie na czas 1 bitu, nadaje pierwszy bit stopu

;	drugi bit stopu
;	goto	$+1				;opoznienie wyrownawcze
;	goto	$+1				;opoznienie wyrownawcze
;	goto	$+1				;opoznienie wyrownawcze
;	call	del_ntb			;opoznienie na czas 1 bitu, nadaje drugi bit stopu

	return

del_ntb		;odmierza opoznienie na czas trwania 1 bitu

	movlw	ntb				;stala do odmierzania czasu 1 bitu	
	movwf	del_reg			;wpisz wartosc poczatkowa do licznika 

	decfsz	del_reg, f		;sprawdz czy del_reg =0 (petla do odliczania opoznienia)
	goto	$-1				;nie, czekaj dalej
	
	return


;================================================================================
;konwersja bin na bcd, wersja 16 bitowa

	;deklaracje rejestrow
	 cblock  
     	r0, r1, r2		    ;liczba dziesietna spakowana (6 cyfr)
     	lbyte, hbyte		;liczba binarna (2 bajty)
	 	cb0, cb1			;liczniki pomocnicze
     endc    

	;r0	dziesiatki 	jednosci
	;r1	tysiace		setki
	;r2	setki tys.	dzies. tys

	;lbyte	mlodszy bajt
	;hbyte	starszy bajt
	
bin2bcd movlw   .16			;liczba bitow
        movwf   cb0			;wpisz do licznika bitow
        clrf    r0
        clrf    r1
        clrf    r2

bin2bcd1 
		movlw   r0
        movwf   fsr			;wskaznik na r0
		movlw	.3			;liczba bajtow wyniku
		movwf	cb1			;wpisz do licznika

bin2bcd2 		;korekta dziesietna
		movlw   h'33'		
        addwf   indf, f		;dodaj h'33'
        btfsc   indf, 3		;sprawdz czy dolny polbajt > 7
		andlw	h'F0'		;tak, zeruj dolny polbajt w
        btfsc   indf, 7		;sprawdz czy gorny polbajt > 7
		andlw	h'0F'		;tak, zeruj gorny polbajt w
        subwf   indf, f		;wynik =< 7, odejmij poprawke
        incf    fsr, f		;nastepny bajt danych
		decfsz	cb1			;nastepne bajty wyniku
        goto    bin2bcd2

		;przesun kolejny bit
		rlf     lbyte, f
        rlf     hbyte, f
        rlf     r0,f
        rlf     r1,f
        rlf     r2,f

        decfsz  cb0, f		;czy wszystkie bity
        goto    bin2bcd1

        retlw   0


;================================================================================


	end



































;PROGRAM ODLEGLOSCIOMIERZ + klawiatura 16-przyciskowa
; by KLAuS :)

	ERRORLEVEL -302
	list P=12F675
	__CONFIG _WDT_OFF & _BODEN_OFF & _MCLRE_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT 
	#include p12f675.inc
	

#define WY GPIO,2
#define SCL TRISIO,4
#define SDA TRISIO,5
#define _BANK STATUS,RP0
#define _C STATUS,0
#define _Z STATUS,2

licznik_i2c equ H'A0'
TXData equ H'A1'
licznik_znak equ H'A2'
kod_znaku equ H'A3'
wyn_MUL1 equ H'A4'
wyn_MUL2 equ H'A5'
wyn_MUL3 equ H'A6'
wyn_MUL4 equ H'A7'
BCD_1 equ H'A8'
BCD_2 equ H'A9'
BCD_3 equ H'AA'
BCD_4 equ H'AB'
TEMP1 equ H'AC'
TEMP2 equ H'AD'
licznik equ H'AE'
licznik_prostokat equ H'AF'
wait1 equ H'B0'
wait2 equ H'B1'

#define out	gpio, 1		;uzywamy GP1
#define nklaw	.16 + 1		;liczba klawiszy +1
	cblock	h'B2'	;adresy rejestrw od xxx
	div		;dzielnik
	xklaw		;nr klawisza
	endc

	org 0x0000
	goto inicjalizacja
	
	org 0x0004
	goto int	;obsluga przerwania

inicjalizacja:
;inicjalizacja procesora
	bcf		_BANK	;bank 0 (GPIO,CMCON)
	clrf	GPIO	;stan wyjsc = zero
	bsf		_BANK ;BANK 1 (TRISIO,VRCON,Option)
	movlw	b'00111111'
	movwf	trisio

	clrf	wpu
	bsf		WPU,5
	bsf		WPU,4	;PODEPNIJ PULL-UPY do lini i2c
	bcf		OPTION_REG,NOT_GPPU	;wlacz Pull-Upy
	clrf	ansel

konfig		;konfiguracja procesora
	bcf		status,rp0
	movlw	b'00010100'	;konfiguruj komparator z vref i wejsciem na gp1
	movwf	cmcon		;cinv=1
	clrf	t1con		;konfiguruj timer 1, presk1=1, clk=tcm	
	bsf		status, rp0	;bank 1
	movlw	b'10001000'	;konfiguruj vref =1/2 vdd, zalacz vref	
	movwf	vrcon
	movlw	b'00111111'	;gp0 - gp2 wejscia
	movwf	trisio

kalibr		;pomiar tm  kalibracja, wykonywany jednokrotnie
	call	roz		;rozladuj kondensator
	clrf	tmr1h
	clrf	tmr1l	;zeruj tmr1l
	call	wait4

	call	lad		;laduj kondensator
	bcf 	status,rp0
	bsf		t1con, tmr1on	;zalacz tmr1

	btfss	cmcon, cout	;sprawdz wyjscie komparatora
	goto	$-1		;czekaj na 1 na wyjsciu komparatora
	bcf		t1con,	tmr1on	;wylacz tmr1
;obliczenie dzielnika div

	bcf		status,rp0
	clrf	div
	movlw	nklaw
div_i
	incf	div, f		;inkrementuj div
	subwf	tmr1l, f	;odejmij nklaw
	btfsc	status, c	;czy roznica ujemna
	goto	div_i		;nie, skok do div_i
;korekta
	addwf	tmr1l, f
	movlw	((nklaw)/.2)	
	subwf	tmr1l, f	;odejmij nklaw/2
	btfss	status, c	;czy roznica ujemna
	decf	div, f		;korekta div

	;koniec kalibracji w rejestrze div aktualna wartosc dzielnika
loop
	clrf	temp1
	incfsz	temp1,f
	call	wait
	
	clrf	temp1
	incfsz	temp1,f
	call	wait
	
	clrf	temp1
	incfsz	temp1,f
	call	wait
	
	clrf	temp1
	incfsz	temp1,f
	call	wait

	clrf	temp1
	incfsz	temp1,f
	call	wait
	
	clrf	temp1
	incfsz	temp1,f
	call	wait

	clrf	temp1
	incfsz	temp1,f
	call	wait

pom_tx		;pomiar_tx i obliczenie xklaw, wykonywany cyklicznie

	call	roz		;rozladuj kondensator
	clrf	tmr1l	;zeruj tmr1l

	call	lad		;laduj kondensator
	bsf		t1con, tmr1on	;zalacz tmr1
	btfss	cmcon, cout	;sprawdz wyjscie komparatora
	goto	$-1		;czekaj na 1 na wyjsciu komparatora
	bcf		t1con,tmr1on	;wylacz tmr1

;obliczenie xklaw
	rrf		div, w		;div/2 -> w
	addwf	tmr1l, f	;dodaj div/2 do tmr1l ze wzgl. na zaokraglenie
	clrf	xklaw		;zeruj xklaw
	movf	div, w		;div -> w
xk_i
	incf	xklaw, f	;inkrementuj div
	subwf	tmr1l, f	;odejmij div
	btfsc	status, c	;czy roznica ujemna
	goto	xk_i		;nie, skok do xk_i
	decf	xklaw, f	;korekta xklaw
				;w rejestrze xklaw numer nacisnietego przycisku

	clrf	wyn_mul2
	movf	xklaw,w
	movwf	wyn_mul1
	call	bcd16
	movlw	h'62'
	call	linia
	call	wyswietl	
	goto 	loop

roz		;rozladowanie
	bcf		status, rp0		;bank 0
	bcf		gpio,gp1		;zeruj gp1
	bsf		status, rp0		;bank 1
	bcf		trisio,1
	bcf		status, rp0	;bank 0
	return

lad		;ladowanie
	bsf		status, rp0		;bank 1
	movlw	b'00111111'		;wcz adowanie
	movwf	trisio
	bcf		status, rp0		;bank 0
	return


	goto $


;podprocedury z przerwania :)
wait:	;czekaj chwilke
	clrf wait1
	clrf wait2
wai2:
	incfsz wait1,f
	goto wai3
	return	;koniec odczekiwania
wai3:
	incfsz wait2,f
	goto $-1
	goto wai2

MUL_16:	;mnozenie dwoch liczb 16-bitowych
	clrf wyn_MUL4	;wyczysc starsza czesc wyniku
	clrf wyn_MUL3	;wyczysc starsza czesc wyniku
	bcf _BANK	;bank 0 (TIMER)
	movf TMR1H,w
	movwf wyn_MUL2	;zawartosc TIMERA 1
	movf TMR1L,w	;przeslij do mlodszej
	movwf wyn_MUL1 	;czesci wyniku

	movlw D'17'	;LICZNIK OBIEGOW PETLI
	movwf licznik
	bcf _C	;czysc bit przeniesienia
MUL_SR: ;przesowa w prawo mnozna
	rrf wyn_MUL4,f
	rrf wyn_MUL3,f
	rrf wyn_MUL2,f
	rrf wyn_MUL1,f	
	decfsz licznik,f	;CZY koniec mnozenia?
	goto TEST_CY
	goto MUL_END	;jesli tak to koniec - RET
TEST_CY:
	btfss _C ;jesli z przesuniecia w prawo powstalo CY to wykonaj dodawanie
	goto MUL_SR	;jesli z przesuniecia w prawo nie bylo CY to przeson w prawo
MUL_SUM:
	movlw H'45'	;POCZATEK procedury DODAWANIA (DLA MNOZENIA)
	addwf wyn_MUL3,f
	btfsc _C
	incfsz wyn_MUL4,f	;przeniesienie :) (tak dziwnie jest w PICach hehe :)
	movlw H'01'	;te smieszne cyfry to wartosc 340
	addwf wyn_MUL4,f ;KONIEC DODAWANIA
	goto MUL_SR	;wykonaj przesuwanie
MUL_END:
	return	; KONIEC mnozenia
;koniec procedury mnozenia 16x16 bit

BCD16
;konwertuje liczbe BIN - wynik mnozenia (3 bajty) -
;w 3 cyfry BCD (bo mi sa tylko 3 potrzebne)
	bcf _C		;zerowanie flagi CY
	clrf BCD_1
	clrf BCD_2
	clrf BCD_3
	movlw .16
	movwf licznik	;licznik obiegow petli
BCD_Loop:
	movlw BCD_1		;adres pierwszej cyfry BCD
	movwf FSR		;do FSR - i adresowanie posrednie :)
	rlf wyn_MUL1,f	;shift BIN
	rlf wyn_MUL2,f
	rlf BCD_1,f		;into BCD
	rlf BCD_2,f
	rlf BCD_3,f
	decfsz licznik,f
	goto DAA	;korekta dziesietna po przesunieciu
	return		;koniec konwersji

DAA:
;procedura robi korekte dziesietna
	movlw .3
	movwf TEMP2	;licznik bajtow BCD do korekty 
DAA_Loop:
	movf INDF,f	;testuj czy obie cyfry BCD nie zero
	btfsc _Z	;jesli tak, to pomin korekte
	goto DAA_end
	movlw H'03'		;mlodsza tetrada
	addwf INDF,W	;korekta mlodszej tetrady
	movwf TEMP1		;zapamietaj wynik posredni
	btfsc TEMP1,3	;czy wynik > 7?
	movwf INDF		;jesli tak to korekta byla potrzebna i zapisz jako nowy wynik
	movlw H'30'		;starsza tetrada
	addwf INDF,W	;korekta starszej tetrady
	movwf TEMP1		;zapamietaj wynik posredni
	btfsc TEMP1,7	;czy wynik > 7?
	movwf INDF		;jesli tak to korekta byla potrzebna i zapisz jako wynik
DAA_end:
	incf FSR,f ;kolejna dwie tetrady BCD
	decfsz TEMP2,f
	goto DAA_Loop
	goto BCD_Loop
;koniec korekty dziesietnej

linia
	movwf	temp1
	call	START_I2C	;zainicjuj magistrale I2C
	movlw	H'7A'	;adres wyswietlacza
	movwf	TXData
	call	WYSLIJ_I2C
	movf	temp1,w	;adres wiersza
	movwf	TXData
	call	WYSLIJ_I2C
	movlw	D'27'	;adres kolumny
	movwf	TXData
	call	WYSLIJ_I2C
	return
wyswietl:
;wyswietlenie wyniku pomiaru
;zaadresowanie wyswietlacza
;	call START_I2C	;zainicjuj magistrale I2C
;	movlw H'7A'	;adres wyswietlacza
;	movwf TXData
;	call WYSLIJ_I2C
;	movlw H'61'	;adres wiersza
;	movwf TXData
;	call WYSLIJ_I2C
;	movlw D'27'	;adres kolumny
;	movwf TXData
;	call WYSLIJ_I2C

	swapf BCD_3,w	;1 cyfra wyniku
	andlw H'0F'
	movwf kod_znaku
	call wysw_znak
	
	movlw H'0F'	
	andwf BCD_3,w	;2 cyfra wyniku
	movwf kod_znaku
	call wysw_znak	

	swapf BCD_2,w	;3 cyfra wyniku
	andlw H'0F'
	movwf kod_znaku
	call wysw_znak
	
	movlw H'0F'	
	andwf BCD_2,w	;4 cyfra wyniku
	movwf kod_znaku
	call wysw_znak
	
	swapf BCD_1,w	;5 cyfra wyniku
	andlw H'0F'
	movwf kod_znaku
	call wysw_znak
	
	movlw H'0F'	
	andwf BCD_1,w	;6 cyfra wyniku
	movwf kod_znaku
	call wysw_znak
	
	call STOP_I2C
	return
;koniec wyswietlania

WYSLIJ_I2C:
;(dana do wyslania ma byc w komorce TXData
;dana jest niszczona
;uzywa komorki 'licznik_i2c'
	movlw	H'08'
	movwf	licznik_i2c	;licznik bitow do wyslania
	bcf		_BANK
	bcf		gpio,4
	bcf		gpio,5
	bsf		_BANK 		;bank 1 (TRISIO)
WW1	bcf		SCL
	rlf		TXData,f	;MSB->Cary
	bcf		SDA			;SData->low
	btfsc	_C			;wyslij jeden bit
	bsf		SDA
	bsf		SCL
	call	wait4
	bcf		SCL			;clk ->low
	decfsz	licznik_i2c,f
	goto	WW1
	bsf		SDA			;czekaj na portwierdzenie
	bsf		SCL
	call	wait4
	bcf		SCL
	bcf		SDA
	return
;koniec procedury wysylajacej bajt I2C

START_I2C:
	bcf		_BANK
	bcf		gpio,4
	bcf		gpio,5
	bsf		_BANK	;bank 1 (TRISIO)
	bsf		SDA
	bsf		SCL
	call	wait4
	bcf		SDA		;(start)
	call	wait4
	bcf		SCL		;INPUT
	call	wait4
	return
;koniec inicjalizacji lini I2C

STOP_I2C:
	bcf		_BANK
	bcf		gpio,4
	bcf		gpio,5
	bsf		_BANK	;bank 1 (TRISIO)
	bcf		SDA
	bsf		SCL
	call	wait4
	bsf 	SDA		;(stop)
	call	wait4
	bcf		SCL		;INPUT
	bcf		SDA
	return
;koniec transmisji I2C
wait4	return		;odczekaj 4us

wysw_znak: 
;wyswietla jeden znak na LCD	
;kod (0-0A) wyswietlanego znaku w komorce 'kod_znaku'
	clrf	licznik_znak	;licznik wyswietlanych kolumn
	clrf	TXData 			;jedna kolumna pusta na wyswietlaczu
	call	WYSLIJ_I2C

	rlf		kod_znaku,f		;mnozenie przez 6
	movf	kod_znaku,w
	rlf		kod_znaku,f		; - bo tyle kolumn ma jeden znak
	addwf	kod_znaku,f		; *6 = *(2+4) = *2 + *4
;	decf	kod_znaku,f
	movlw	h'01'			;ustaw strone w pamieci dla tablicy
	movwf	PCLATH			;takie cuda sa konieczne w PICach :(
wz1
	movf	licznik_znak,w	;sprawdzanie czy wyswietlono caly znak
	sublw	.6	
	btfsc	status,z		;czyli 6 kolumn z matrycy
	goto	wz2				;jesli tak to wyswietl jedna kolumne rozdzielajaca
	movf	kod_znaku,w
	addwf	licznik_znak,w
	call	maska_znak
	movwf	TXData
	call	WYSLIJ_I2C
	incf	licznik_znak,f
	goto	wz1				;wyswietlono jeden znak
wz2	clrf	TXData				;jedna kolumna pusta na wyswietlaczu
	call	WYSLIJ_I2C
	return
;koniec wysylania jednego pelnego znaku	

int:
;procedura obslugi przerwan
	bcf T1CON,TMR1ON	;wylacz timera
	retfie			;koniec przerwania
	
overflow:
;obsluga przekroczenia zakresu
	bcf PIR1,TMR1IF	;wyczysc znacznik przerwania
	movlw H'0A'		;symbol niepoprawnego wyniku
	movwf BCD_4
	movlw H'AA'		;symbol niepoprawnego wyniku
	movwf BCD_3
	call wyswietl	;wyswietl na wyswietlaczu
	bcf _BANK		;bank 0 (INTCON)
	bcf INTCON,PEIE	;zablokuj chwilowo przerwania od peryferiow
	retfie

;tablica znakow dla wyswietlacza
	org 150
maska_znak:	;'w' przed wywolaniem zawiera kod_znaku + licznik (czyli ktora kolumna)
	addwf PCL,f	;odczytaj z tablicy maske znaku
	dt H'7E',H'81',H'81',H'81',H'81',H'7E' ;maska dla '0' 
	dt H'00',H'84',H'82',H'FF',H'80',H'00' ;maska dla '1' 
	dt H'82',H'C1',H'A1',H'91',H'89',H'86' ;maska dla '2' 
	dt H'41',H'81',H'81',H'85',H'8B',H'71' ;maska dla '3' 
	dt H'30',H'28',H'24',H'22',H'FF',H'20' ;maska dla '4' 
	dt H'4F',H'89',H'89',H'89',H'89',H'71' ;maska dla '5'	 
	dt H'7C',H'8A',H'89',H'89',H'89',H'70' ;maska dla '6' 
	dt H'03',H'01',H'01',H'F1',H'09',H'07' ;maska dla '7' 
	dt H'76',H'89',H'89',H'89',H'89',H'76' ;maska dla '8' 
	dt H'0E',H'91',H'91',H'91',H'51',H'3E' ;maska dla '9' 
	dt H'80',H'80',H'80',H'80',H'80',H'80' ;maska dla 'overflow' '-'
	dt H'00',H'00',H'C0',H'C0',H'00',H'00' ;maska dla '.'
	dt H'F8',H'08',H'08',H'F0',H'08',H'F8' ;maska dla 'm'  	

napis: ;napis 'odleglosc:'
	addwf PCL,f	;odczytaj z tablicy maske znaku
	dt H'24',H'24',H'00'
	dt H'00',H'20',H'45',H'46',H'44',H'38',H'00'
	dt H'00',H'20',H'55',H'56',H'54',H'48',H'00'
	dt H'00',H'38',H'44',H'44',H'44',H'38',H'00'
	dt H'00',H'00',H'44',H'7F',H'51',H'00',H'00'
	dt H'00',H'3E',H'52',H'52',H'52',H'0C',H'00'
	dt H'00',H'18',H'54',H'54',H'54',H'38',H'00'
	dt H'00',H'00',H'40',H'7F',H'41',H'00',H'00'
	dt H'00',H'7F',H'48',H'44',H'44',H'38',H'00'
	dt H'00',H'38',H'44',H'44',H'44',H'38'

	end
