;procedury i2c

;===================================================================================
;listing 7.17 definiowanie rejestrow i oznaczen dla procedury obslugi interfejsu i2c
;===================================================================================

i2c_def		;definicje rejestrow i oznaczen

i2c_reg		equ	h'20'
i2c_state	equ	h'21'
licznik		equ	h'21'

#define		sda	gpio, 4
#define		scl	gpio, 5

#define		ack	i2c_state, 0

#define		read	iorlw	b'00000001'		;ustawia bit r/w
#define		write	andlw	b'11111110'		;zeruje  bit r/w

#define		bank0	bcf	status, rp0
#define		bank1	bsf	status, rp0

#define		ze	status, z					;flaga z
#define		ca	status, c					;flaga c

	return



;========================================================================
;listing 7.18 procedura inicjalizacji rejestrow portu dla linii sda i scl
;========================================================================

i2c_port_ini	;inicjalizacja rejestrow portow
	
	bank0
	bcf		sda			;przygotuj 0 na sda
	bcf		scl			;przygotuj 0 na scl
	
	bank1
	bsf		sda			;linia sda - wejscie
	bsf		scl			;linia scl - wejscie
	
	bank0
	
	return


;========================================================================
;listing 7.19 zapisywanie stanow 0 albo 1 na linie sda i scl
;========================================================================

	bank1
	bcf	sda	;0 na linie sda
	bsf	sda	;1 na linie sda i przygotowanie linii sda do odczytu
	
	bcf	scl	;0 na linie scl
	bsf	scl	;1 na linie scl i przygotowanie linii scl do odczytu
	bank0


;========================================================================
;listing 7.20 procedura odczytu stanu linii sda
;========================================================================

	bank1
	bsf		sda			;przygotowanie linii sda do odczytu
	bank0
	bsf		ca			;1 -> ca
	btfss	sda			;sprawdz sda
	bcf		ca			;sda = 0, 0 -> ca
	rlf		i2c_reg, f	;wpisz bit do i2c_re


;========================================================================
;listing 7.21 procedura odczytu stanu linii scl i wykrywanie 
;stanu oczekiwania (wait state)
;========================================================================

	bank1
	bsf		scl		;wpisz 1 na scl
	bank0
	btfss	scl		;sprawdz scl
	goto	$-1		;scl=0, czekaj


;========================================================================
;listing 7.22 zapis na linie sda stanu z rejestru i2c_reg
;========================================================================

	bank0
	rlf		i2c_reg	, f	;przesun bit do flagi c
	bank1
	btfss	ca		;sprawdz c
	bcf		sda		;c=0, 0->sda
	btfsc	ca		;sprawdz c
	bsf		sda		;c=1, 1->sda
	bank0


;========================================================================
;listing 7.23 procedura start - inicjalizacja transmisji
;========================================================================

i2c_start		;wysyla sekwencje start
	bank1
	bcf		sda		;0->sda
	
	goto	$+1		;opoznienie
	goto	$+1
	goto	$+1
	
	bcf		scl		;0->scl
	bank0
	
	return

;========================================================================
;listing 7.24 procedura stop - zakonczanie transmisji
;========================================================================

i2c_stop		;wysyla sekwencje stop
	bank1
	bsf		scl		;wpisz 1 na scl
	bank0
	btfss	scl		;sprawdz scl
	goto	$-1		;scl=0, czekaj
	
	bank1
	bsf		sda		;1->sda
	bank0
	
	return


;========================================================================
;listing 7.25 procedura wyslania adresu lub danych
;========================================================================

i2c_send			;wysyla adres lub dane z rejestru w
		
	bank0
	movwf	i2c_reg	;wpisz dane do rejestru i2c_reg
	
	movlw	.8		;liczba bitow
	movwf	licznik
	
send
	rlf	i2c_reg, f	;przesun bit do c
	bank1
	btfss	ca		;sprawdz c
	bcf		sda		;c=0, 0->sda
	btfsc	ca		;sprawdz c
	bsf		sda		;c=1, 1->sda
		
	bsf		scl		;wpisz 1 na scl
	bank0
	btfss	scl		;sprawdz scl
	goto	$-1		;scl=0, czekaj
	goto	$+1		;opoznienie
	bank1
	bcf		scl		;wpisz 0 na scl
	bank0
	decfsz 	licznik	;sprawdza liczbe wyslanych bitow
	goto	send	;wyslij kolejny bit
	
	goto	$+1		;opoznienie
	goto	$+1		;opoznienie
	
	bank1
	bsf		sda		;przygotowanie linii sda do odczytu
		
		;sprawdza ack
	
	bsf		scl		;wpisz 1 na scl
	bank0
	btfss	scl		;sprawdz scl
	goto	$-1		;scl=0, czekaj
		
	bank0
	bsf		ack
	btfss	sda		;czy ack=1
	bcf		ack		;ack=0
	bank1
	bcf		scl		;wpisz 0 na scl
	bank0
	
	return


;========================================================================
;listing 7.26 procedura odbioru danych
;========================================================================

i2c_rec				;odbiera dane do rejestru i2c_reg
	;flaga porwierdzenia ack=0 wysyla potwierdzenie
	;flaga porwierdzenia ack=1 nie wysyla potwierdzenia
	
	bank1
	bsf		sda		;przygotowanie linii sda do odczytu
	
	bank0
	movlw	.8		;liczba bitow
	movwf	licznik
	
	bank1
rec			;odbiera 8 kolejnych bitow
	bsf		scl		;wpisz 1 na scl
	bank0
	btfss	scl		;sprawdz scl
	goto	$-1		;scl=0, czekaj
	
	bank0
	bsf		ca
	btfss	sda
	bcf		ca
	rlf		i2c_reg, f
	
	bank1
	bcf		scl		;wpisz 0 na scl
	bank0
	decfsz 	licznik	;sprawdza liczbe odebranych bitow
	goto	rec		;odbierz kolejny bit
	
	goto	$+1		;opoznienie
	bank1
	btfss	ack		;sprawdz ack
	bcf		sda		;wyslij potwierdzenie
	
	goto	$+1		;opoznienie
	bsf		scl		;wpisz 1 na scl
	bank0
	btfss	scl		;sprawdz scl
	goto	$-1		;scl=0, czekaj
	
	goto	$+1		;opoznienie
	
	bank1
	bcf		scl		;wpisz 0 na scl
	goto	$+1		;opoznienie
	bsf		sda		;zwolnienie linii sda 
	bank0
	
	return


;========================================================================
;listing 7.27 procedura wyslania danych do urzadzenia
;========================================================================

send_data	;wysyla dane do urzadzenia
			;adres urzadzenia w rejestrze adr_reg

	call	port_ini
	
	call 	start		;wyslij start
	movf	adr_reg, w	;wpisz adres do w
	write				;zapis do slave, zeruje bit r/w 
	
	call	i2c_send	;wyslij adres
	
	btfsc	ack			;sprawdz potwierdzenie
	goto 	send_end	;brak poptwierdzenia
						;jest potwierdzenie
	
	movf	data, w		;wpisz dane do w
	call	i2c_send	;wyslij dane
	
	btfsc	ack			;sprawdz potwierdzenie
	goto 	send_end	;brak poptwierdzenia
						;jest potwierdzenie
	
	;wyslij nastepne dane
	
send_end				;koniec procedury
	call stop			;wyslij stop
	
	return
	

;========================================================================
;listing 7.27 procedura odbioru danych z urzadzenia
;========================================================================
	
read_data	;odbiera dane z urzadzenia
			;adres urzadzenia w rejestrze adr_reg
	
	call	port_ini	;inicjalizacja portow
	
	call 	start		;wyslij start
	movf	adr_reg, w	;wpisz adres do w
	read				;odczyt ze slave, ustawia bit r/w 	

	call	i2c_send	;wyslij adres
	
	btfsc	ack			;sprawdz potwierdzenie
	goto 	read _end	;brak poptwierdzenia
						;jest potwierdzenie
	
	;odbierz dane
	
	bcf		ack			;wysylaj potwierdzenie
	call	i2c_read	;odbierz dane
	
	;odbierz nastepne dane
	
	bsf		ack			;nie wysylaj potwierdzenia
	call	i2c_read	;odbierz dane
	
read_end				;koniec procedury
	call 	stop		;wyslij stop
	
	return


;========================================================================	




