;procedury one_wire
;co 1 sek odczyt pamieci ROM ukladu 1_wire (rozkaz 33h)
;zapala led na 0.5 sek jezeli zgadza sie suma crc i nr seryjny
;zapisuje nr seryjny ukladu do pamieci EEPROM procesora



	ERRORLEVEL -302
	list P=12F675
	__CONFIG _WDT_OFF & _BODEN_OFF & _MCLRE_ON & _PWRTE_ON & _INTRC_OSC_NOCLKOUT 
	#include p12f675.inc
	

;Konfiguracja procesora
	
;Konf_ow	;konfiguracja procesora i definicje

#define dq		gpio, gp5				;definicja linii dq
#define out		gpio, gp2				;definicja linii out


#define bank0	bcf	status,rp0		;definicja bank0
#define bank1	bsf	status,rp0		;definicja bank1

#define pr_ok	state, 0			;definicja flagi pr_ok
#define crc_ok	state, 1			;definicja flagi crc_ok.
#define sn_ok	state, 2			;definicja flagi sn_ok.

;numer seryjny ukladu DS1990
;00 00 08 F8 87 C2

#define	sn0	h'C2'					;zerowy bajt sn
#define	sn1	h'87'					;pierwszy bajt sn 
#define	sn2	h'F8'					;drugi bajt sn
#define	sn3	h'08'					;trzeci bajt sn
#define	sn4	h'00'					;czwarty bajt sn
#define	sn5	h'00'					;piaty bajt sn

	cblock	h'28'
	;adresy 20h do 27h zarezerwowane na odczytane dane
	;zapis calej ramki 1-wire 8 bajtow
	
	state			;rejestr flag
	dat				;rejestr na dane
	crc				;rejestr crc
	cnt, cnt1, cnt2	;liczniki 
	
	endc

	org 0
	goto	start

;========================================================================
start	
	
	bank1
	movlw	b'00101000'		;niewykorzystane linie procesora (GP0, GP1, GP4) = wyjscia	
	movwf	trisio			;GP3 - zerowanie procesora przyciskiem
	bank0

	;przygotowanie linii procesora

	Bcf		out				;zeruj out
	Bcf		dq				;0 -> dq
	
	bank1					;bank 1
	Bsf		dq				;1 -> tris5
	Bcf		out				;0 -> tris2		
	bank0					;bank 0

	clrf	state


;========================================================================
;program glowny 

loop		;petla glowna

	call 	reset			;sekwencja inicjalizacji
	btfss	pr_ok			;sprawdz presence
	goto	wait_1s			;nie ma presence, czekaj 1 sek
	movlw	h'33'			;jest, wpisz do w kod rozkazu 33h - czytaj ROM
	call	sd_cmd			;wyslij rozkaz 
	call	read_frame		;odczytaj dane

;	call	podstaw			;procedura testowa - zapis znanego nr seryjnego do pamieci ram

	call	test_crc		;oblicz crc
	btfss	crc_ok			;sprawdz crc_ok
	goto	wait_1s			;zly, czekaj 1 sek i nowa inicjalizacja
	
;	call	test_sn			;odczytaj numer seryjny
;	btfss	sn_ok			;sprawdz sn_ok	
;	goto	wait_1s			;zly, czekaj 1 sek i nowa inicjalizacja
	
		;sprawdzenia ok, zapal diode na 0.5 sek

	call	zapis_ee		;zapisz nr seryjny ukladu do ee

	bsf		out				;zapal diode
	call 	wait_500		;czekaj 0.5 sek	
	bcf		out				;zgas diode
	goto	wait_05s		;czekaj 0.5 sek

	;... inne procedury wykonywane po poprawnej identyfikacji
	
wait_1s		;czekaj 1 sek	
	call 	wait_500		;czekaj 0.5 sek
wait_05s	;czekaj 0.5 sek
	call 	wait_500		;czekaj 0.5 sek
	
	Goto	loop			;idz do petli
	
	
;========================================================================
;========================================================================	
;procedury
;========================================================================
;========================================================================

;sekwencje zwolnienia i zerowania magistrali wykorzystywane w programie

;zwolnij magistrale
;	bank1					;bank 1
;	Bsf		dq				;1 -> dq - zwolnij magistrale
;	bank0					;bank 0

;zeruj magistrale
;	bank0
;	Bcf		dq				;0 -> dq - przygotuj 0 na dq
;	bank1					;bank 1
;	Bcf		dq				;0 -> dq - wpisz 0 ma magistrale
;	bank0					;bank 0


;========================================================================
reset	;sekwencja inicjalizacji

	bank1					;bank 1
	Bsf		dq				;1 -> dq - zwolnij magistrale
	bank0					;bank 0
	
	bcf		pr_ok			;zeruj flage presence
	
	bank0
	Bcf		dq				;0 -> dq - przygotuj 0 na dq
	bank1					;bank 1
	Bcf		dq				;0 -> dq - wpisz 0 ma magistrale
	bank0					;bank 0
	
	Call	del_500			;opoznienie 500 us (> 480 us)
	
	bank1					;bank 1
	Bsf		dq				;1 -> dq - zwolnij magistrale
	bank0					;bank 0

	Call	del_70			;opoznienie 70 us (> 40 us, < 240 us)
							
	btfss	dq				;sprawdz presence 
	bsf		pr_ok			;jest presence, ustaw flage	presence

	Call	del_430			;opoznienie 430 us (70+430 > 480 us)

	btfss	dq				;sprawdz  linie dq po ok 1000 us
	bcf		pr_ok			;nie zwolniona, zeruj flage	presence

	
	return


;========================================================================
sd_cmd			;wyslij rozkaz, kod rozkazu jest w rejestrze w
	
	movwf	dat				;w -> dat, dane do wyslania

	movlw	.8				;liczba bitow
	movwf	cnt				;licznik bitow

write_bit					;zapis 1 bitu

	bank0
	Bcf		dq				;0 -> dq - przygotuj 0 na dq
	bank1					;bank 1
	Bcf		dq				;0 -> dq - wpisz 0 ma magistrale
	bank0					;bank 0

	Call	del_5			;opoznienie 5 us
	
	Rrf		dat, f			;wysun bit do c
	
	Btfsc	status, c		;sprawdz flage c
	Bsf		dq				;1 -> dq, wpisz 1 na wyjscie jesli c=1
	
	Call	del_60			;opoznienie 60 us (razem z poprzednimi op. > 60 us, < 120 us)
	
	bank1					;bank 1
	Bsf		dq				;1 -> dq - zwolnij magistrale
	bank0					;bank 0
		
	goto	$+1				;opoznienie 2 us
	goto	$+1				;opoznienie 2 us
		
	Decfsz	cnt, f			;czy wszystkie bity
	Goto	write_bit		;zapisz kolejny bit	
	
	Return
	
	
;========================================================================
read_frame		;odczyt ramki danych - 8 bitow
				;Bufor danych w RAM, 8 bajtow, adresy 20h  27h, 
			
	movlw 	.8				;liczba danych
	movwf	cnt1			;licznik danych

	movlw	h'20'			;adres pierwszego bajtu
	movwf	fsr				;wpisz do fsr
	
read_loop		;petla do odbioru ramki (8 bajtow)

	call 	read_byte		;odczytaj bajt
	movf	dat, w			;dat -> w
	movwf	indf			;zapisz do bufora
	incf	fsr, f			;adres nastepnego bajtu
	decfsz 	cnt1, f			;czy wszystkie dane
	goto 	read_loop		;nie, odbieraj dalej
	
	return
	
	
;========================================================================	
read_byte		;odczyt 1 bajtu danych z linii dq
	
	movlw	.8				;liczba bitow
	movwf	cnt				;licznik bitow

read_bit	;odczyt 1 bitu

	bank0
	Bcf		dq				;0 -> dq - przygotuj 0 na dq
	bank1					;bank 1
	Bcf		dq				;0 -> dq - wpisz 0 ma magistrale
	bank0					;bank 0
	
	Call	del_5			;opoznienie 5 us
	
	bank1					;bank 1
	Bsf		dq				;1 -> dq - zwolnij magistrale
	bank0					;bank 0
	
	goto	$+1				;opoznienie 2 us
	nop						;opoznienie 1 us
			
	Bcf		status, c
	Btfsc	dq				;sprawdz dq (odczytaj bit)
	Bsf		status, c
	Rrf		dat, f			;zapisz bit
	
	Call	del_50			;opoznienie 50 us (razem z poprzednimi op. > 60 us, < 120 us)
	
	Decfsz	cnt, f			;czy odebrano 8 bitow
	Goto	read_bit		;odbieraj dalej
	
	return


;========================================================================	
test_crc	;oblicza crc dla calej ramki (8 bajtow danych)
	
	movlw	h'20'			;adres pierwszego bajtu
	movwf	fsr				;wpisz do fsr

	movlw 	.8				;liczba bajtow
	movwf	cnt1			;licznik bajtow
	
	clrf	crc				;zeruj crc

crc_loop		;petla do obliczenia crc z 8 bajtow

	movf	indf, w			;wpisz dane do w
	incf	fsr, f			;adres nastepnego bajtu
	call	obl_crc			;oblicz crc dla 1 bajtu
	decfsz 	cnt1, f			;czy wszystkie bajty
	goto 	crc_loop		;nie, obl crc dla kolejnego bajtu
	
		;sprawdza crc dla calosci
	movf	crc, f			;przeczytaj crc
	bcf		crc_ok			;zeruj crc_ok
	btfsc	status, z		;czy rowne 0
	bsf		crc_ok			;ustaw crc_ok
	
	return


;========================================================================
obl_crc	;oblicza crc8 dla 1 bajtu na podstawie, wielomian=x^8+x^5+x^4+1
		;dane w dat, wynik w crc
		;przed pierwszym wywolaniem wyzerowac crc
		
	movwf	dat				;w -> dat	

	movlw	.8				;liczba bitow
	movwf	cnt				;licznik bitow

crc0	
	rrf 	dat, f
	btfsc	status, c	
	goto	crc3
	btfsc	crc, 7
	goto	crc4

crc1
	bcf		status, c
	rlf		crc, f
crc2
	decfsz	cnt, f
	goto	crc0
	
	return

crc3
	btfsc	crc, 7	
	goto	crc1
crc4
	movlw	18h
	xorwf	crc, f
	bsf		status, c
	rlf		crc, f
	goto 	crc2


;========================================================================
test_sn	;sprawdz numer seryjny
	;odczyt numeru seryjnego z pamieci programu
	;porownanie z numerem w pamieci RAM
	;kolejne bajty odczytanego sn maja adresy 21h  27h
	
	movlw	.6				;liczba bajtow sn
	movwf	cnt1			;licznik bajtow sn

	bcf		sn_ok			;zeruj sn_ok.

	movlw	21h				;adres bajtu 1 w pamieci RAM
	movwf	fsr				;wpisz do fsr
	
sn_rd		;odczyt i porownanie kolejnych bajtow sn

	movlw	21h				;adres bajtu 1 w pamieci RAM
	subwf	fsr,w			;oblicz adres bajtu 1 w tablicy
	call	tab_sn			;odczytaj bajt sn (w=snx)
	xorwf	indf, w			;porownaj bajty
	btfss	status, z		;sprawdz czy zgodne
	return					;nie zgodne	

	incf	fsr, f			;zgodne, nastepny bajt
	decfsz 	cnt1,f			;czy wszystkie bajty
	goto 	sn_rd			;czytaj kolejny bajt sn

	bsf		sn_ok			;wszystkie bajty ok. ustaw sn_ok
	return

;========================================================================

tab_sn	;tablica z numerem seryjnym, zwraca bajty  sn w rejestrze w
	addwf	pcl, f						;skok wyliczany
	dt	sn0, sn1, sn2, sn3, sn4, sn5 	;sn w pamieci programu 


;========================================================================
;opoznienia dla sekwencji 1-wire


Del_5						;opoznienie 5 us,(fclk=4MHz)
	nop
	Return

Del_10						;opoznienie 10 us,(fclk=4MHz)
	Movlw	.1
	Goto	del_t

Del_50						;opoznienie 50 us,(fclk=4MHz)
	Movlw	.15
	Goto	del_t

Del_60						;opoznienie 60s,(fclk=4MHz)
	Movlw	.18
	Goto	del_t

Del_70						;opoznienie 70s,(fclk=4MHz)
	Movlw	.22
	Goto	del_t
	
Del_430						;opoznienie 430,(fclk=4MHz)
	Movlw	.142
	Goto	del_t

Del_500						;opoznienie 500,(fclk=4MHz)
	Movlw	.165
	Goto	del_t


Del_t						;petla opozniajace
	Movwf	cnt2			;inicjalizacja cnt2
	
	Decfsz	cnt2, f			;petla wewn 3 cykle maszynowe
	Goto	$-1				;czekaj w petli

	Return


;========================================================================
wait_500	;opoznienie 500 ms, (fclk=4MHz)

	bank0

	movlw	b'00110000'		;wewn. clk, presk. =8
	movwf	t1con			;konfiguracja tmr1

	clrf	tmr1l			;zeruj tmr1l	
	clrf	tmr1h			;clrf tmr1h
	bcf		pir1, t1if		;zeruj t1if
	bsf		T1CON, TMR1ON	;zalacz tmr1

	btfss	pir1, t1if		;sprawdz t1if
	goto	$-1				;czekaj na ustawienie t1if
	bcf		t1con, tmr1on	;wylacz tmr1
	bcf		pir1, t1if		;zeruj t1if

	return


;========================================================================
podstaw		;procedura testowa

;zamiast procedury odbioru wpisuje do pamieci dane z ukladu DS1990
;1E 00 00 08 F8 87 C2 01

	movlw	h'01'
	movwf	h'20'
	movlw	h'C2'
	movwf	h'21'
	movlw	h'87'
	movwf	h'22'
	movlw	h'F8'
	movwf	h'23'
	movlw	h'08'
	movwf	h'24'
	movlw	h'00'
	movwf	h'25'
	movlw	h'00'
	movwf	h'26'
	movlw	h'1E'
	movwf	h'27'

	return


;========================================================================
zapis_ee	;zapisuje 8 bajtow do pamieci ee

	movlw	.8				;liczba bajtow
	movwf	cnt1			;licznik bajtow
	
	movlw	h'20'			;adres pierwszego bajtu
	movwf	fsr				;wpisz do fsr
	
	bank1
	clrf	eeadr			;adres pierwszego bajtu w ee
	bank0
	
zapis_loop					;petla do zapisu kolejnych bajtow

	movf	indf, w			;dane do zapisu -> w
	call	eewrite			;zapisz 1 bajt
	
	incf	fsr, f			;adres kolejnego bajtu w pamieci ram
	bank1
	incf	eeadr, f		;adres kolejnego bajtu w pamieci ee
	bank0
	
	decfsz	cnt1, f			;czy wszystkie bajty
	goto	zapis_loop		;zapisz kolejny bajt
	
	return


;========================================================================
eewrite		;zapis bajtu do pamieci eeprom

	bank1					;bank 1
	movwf	eedata			; w -> eedata
	
	bsf	eecon1,wren			;zezwolenie na zapis
	bcf	intcon,gie			;wylaczenie przerwan

	;sekwencja zapisu
	movlw	h'55'					
	movwf	eecon2						
	movlw	h'AA'						
	movwf	eecon2						
	bsf		eecon1,wr		;rozpoczecie zapisu		
	
	btfsc	eecon1, wr		;czy zakonczony zapis
	goto	$-1				;czekaj w petli

	bcf		eecon1,wren		;wylaczenie zezwolenia na zapis
	bank0					;bank 0


	return


;========================================================================

	end


