;test_nad_rs - test nadawania przez rs
;po zalaczeniu nadaje 0 1 2 3 4 5 6 7 8 9 co 0.5 sek
;sygnalizuje dioda led nadawanie sekwencji


;	GP5 - linia out - (wyjscie TX RS232)
;	GP2	- led  - polaczona do GP2

#define out 	gpio, 5		;deklaracja linii out (GP5)		
#define led		gpio, 2		;deklaracja linii led (GP2)	

#define baud	.9600		;deklaracja szybkosci transmisji w bitach/sek
	

;===============================================================
;wartosci stalych ntb i ptb dla roznych szybkosci transmisji 
;najblizsza wartosc dla Fclk = 4.000 MHz
;uwaga - nalezy skalibrowac wewnetrzny oscylator
;ntb = (((fclk/4)/baud)-13)/3, fclk w Hz, baud w bitach/sek

	;TABELA ntb i ptb (wartosci dziesietne)

	; 2400 bit/sek - ntb = .134 lub .135	ptb = .66
	; 4800 bit/sek - ntb = .65 				ptb = .31
	; 9600 bit/sek - ntb = .30 lub .31		ptb = .14
	;19200 bit/sek - ntb = .13				ptb =  .5
	;38400 bit/sek - ntbt = .4				ptb =  .1


	if	baud == .2400		;szybkosc 2400 bit/sek
	#define	ntb	.134		;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .4800		;szybkosc 4800 bit/sek
	#define	ntb	.65			;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .9600		;szybkosc 9600 bit/sek
	#define	ntb	.30			;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .19200		;szybkosc 19200 bit/sek
	#define	ntb	.13			;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .38400		;szybkosc 38400 bit/sek
	#define	ntb	.4			;stala do odmierzania czasu trwania 1 bitu 
	endif

	ifndef	ntb
	error	"Zla szybkosc transmisji"
	endif


;===============================================================

	cblock	h'20'			;deklaracje adresow rejestrow

	tx_reg					;rejestr szeregowy nadajnika rs
	cnt						;licznik bitow
	del_reg					;licznik opoznienia

	endc

;=============================================================== 

 	;deklaracja typu procesora i formatu liczb

	list p=12F675			;okreslenie typu procesora
	#include p12f675.inc	;dolaczenie pliku z nazwami

	;konfiguracja procesora, fclk = 4 MHz
	__config _INTRC_OSC_NOCLKOUT &_MCLRE_ON &_WDT_OFF

	errorlevel -302			;nie wyprowadzaj komunikatu nr 302
	
	org	h'00'				;adres poczatku programu
	goto	start			;skok do poczatku programu

	
start		;pocztek programu

	;konfiguracja procesora

	;wylacz komparator
	movlw	.7				;wylacz komparator
	movwf	cmcon			;cinv=1

	;wylacz A/C
	bsf		status, rp0		;bank1
	clrf	ansel

	;fabryczna kalibracja oscylatora
	call	h'3ff'			;fabryczna wartosc kalibracyjna -> w
	movwf	osccal			;w -> osccal
	bcf		status, rp0		;bank0

	movlw	b'00100000'		;inicjalizacja linii portu
	movwf	gpio

	bsf	status, rp0			;bank 1

	;konfiguracja linii procesora
	movlw	b'00001000'		;wejscie GP3, reszta to wyjscia
	movwf	trisio

	bcf		status, rp0		;bank 0

	;konfiguracja licznika 1
	movlw	b'00110000'		;preskaler = 8, wewn. zegar 
	movwf	t1con

	clrf	tmr1h			;zeruj rejestr tmr1h
	clrf	tmr1l			;zeruj rejestr tmr1l

	clrf	intcon			;zablokuj przerwania
	clrf	pir1			;zeruj flagi - t1if

	bsf		t1con, tmr1on	;zalacz timer 1




;===============================================================
loop		;petla glowna

	btfss	pir1, t1if		;sprawdz flage t1if
	goto	loop			;czekaj na przepelnienie timera 1
							;nastapilo przepelnienie timera 1 ( co 0.524 sek)
	bcf		pir1, t1if		;zeruj flage przepelnienia
	
	bsf		led				;zapal led
	
;nadaje 0, 1 ... 9

	movlw	h'30'		;0
	call	tx_out

	movlw	h'31'		;1
	call	tx_out

	movlw	h'32'		;2 
	call	tx_out

	movlw	h'33'		;3
	call	tx_out

	movlw	h'34'		;4
	call	tx_out

	movlw	h'35'		;5
	call	tx_out

	movlw	h'36'		;6
	call	tx_out

	movlw	h'37'		;7
	call	tx_out

	movlw	h'38'		;8
	call	tx_out

	movlw	h'39'		;9
	call	tx_out


	movlw	h'0A'		;nowa linia
	call	tx_out

	bcf		led			;zgas led

	goto 	loop		;powrot do petli			


;================================================================================
;procedury
;================================================================================


;================================================================================
tx_out		;wysyla bit startu, 8 bitow danych i bit stopu

	movwf	tx_reg			;wpisz dane do rejestru tx_reg

	movlw	.9
	movwf	cnt				;licznik bitow (1+8)

	bcf	status, c			;wpisz bit startu

tx_loop		;petla do nadwania bitow
	
	btfss	status, c		;sprawdz c
	bcf		out				;c=0, zeruj out
	btfsc	status, c		;sprawdz c
	bsf		out				;c=1, ustaw out

	call	del_ntb			;opoznienie na czas 1 bitu
	
	rrf		tx_reg, f		;przesun do c kolejny bit

	decfsz	cnt, f			;sprawdz czy nadano wszystkie bity
	goto	tx_loop			;nie, nadawaj kolejny bit

	bsf	status, c			;wpisz bit stopu

	btfss	status, c		;sprawdz c
	bcf		out				;c=0, zeruj out
	btfsc	status, c		;sprawdz c
	bsf		out				;c=1, ustaw out

	goto	$+1				;opoznienie wyrownawcze
	goto	$+1				;opoznienie wyrownawcze
	call	del_ntb			;opoznienie na czas 1 bitu, nadaje pierwszy bit stopu

;	drugi bit stopu
;	goto	$+1				;opoznienie wyrownawcze
;	goto	$+1				;opoznienie wyrownawcze
;	goto	$+1				;opoznienie wyrownawcze
;	call	del_ntb			;opoznienie na czas 1 bitu, nadaje drugi bit stopu

	return


;================================================================================
del_ntb		;odmierza opoznienie na czas trwania 1 bitu

	movlw	ntb				;stala do odmierzania czasu 1 bitu	
	movwf	del_reg			;wpisz wartosc poczatkowa do licznika 

	decfsz	del_reg, f		;sprawdz czy del_reg =0 (petla do odliczania opoznienia)
	goto	$-1				;nie, czekaj dalej	

	return
;===============================================================================


	end
