;term_3k termometr 3 kanalowy
;wysyla co 0.5 sek. wyniki pomiaru temperatury z 3 czujnikow TC1047A przez rs
;zakres temperatur -40 - +125 'C, rozdzielczosc 0.25 'C


;	GP5 - linia out - (wyjscie TX RS232)
;	GP4	- kanal 3
;	GP3	- przycisk zerowania, wymagany rezystor podciagajacy do +Vcc (ok. 20 k)
;	GP2	- kanal 2
;	GP1	- wejscie Vref (2.500 V)
;	GP0	- kanal 0

#define out 	gpio, 5		;deklaracja linii out (GP5)		

#define baud	.9600		;szybkosci transmisji w bitach/sek

#define	zero	.205		;zero termometru (kwanty przetw. a/c dla vref = 2.500 V)

#define	minus	znak, 0		;flaga ujemnej temperatury	

;===============================================================
;wartosci stalych ntb i ptb dla roznych szybkosci transmisji 
;najblizsza wartosc dla Fclk = 4.000 MHz
;uwaga - nalezy skalibrowac wewnetrzny oscylator
;ntb = (((fclk/4)/baud)-13)/3, fclk w Hz, baud w bitach/sek

	;TABELA ntb i ptb (wartosci dziesietne)

	; 2400 bit/sek - ntb = .134 lub .135	ptb = .66
	; 4800 bit/sek - ntb = .65 				ptb = .31
	; 9600 bit/sek - ntb = .30 lub .31		ptb = .14
	;19200 bit/sek - ntb = .13				ptb =  .5
	;38400 bit/sek - ntbt = .4				ptb =  .1


	if	baud == .2400		;szybkosc 2400 bit/sek
	#define	ntb	.134		;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .4800		;szybkosc 4800 bit/sek
	#define	ntb	.65			;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .9600		;szybkosc 9600 bit/sek
	#define	ntb	.30			;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .19200		;szybkosc 19200 bit/sek
	#define	ntb	.13			;stala do odmierzania czasu trwania 1 bitu 
	endif

	if	baud == .38400		;szybkosc 38400 bit/sek
	#define	ntb	.4			;stala do odmierzania czasu trwania 1 bitu 
	endif

	ifndef	ntb
	error	"Zla szybkosc transmisji"
	endif


;===============================================================

	cblock	h'20'			;deklaracje adresow rejestrow

	tx_reg					;rejestr szeregowy nadajnika rs
	tx_cnt						;licznik bitow nadajnika rs
	del_reg					;licznik opoznienia nadajnika rs

	nr_kan					;numer kanalu
	ad_h, ad_l				;rejestry na wynik MSB i LSB

	endc

;=============================================================== 

 	;deklaracja typu procesora i formatu liczb

	list p=12F675			;okreslenie typu procesora
	#include p12f675.inc	;dolaczenie pliku z nazwami

	;konfiguracja procesora, fclk = 4 MHz
	__config _INTRC_OSC_NOCLKOUT &_MCLRE_ON &_WDT_OFF

	errorlevel -302			;nie wyprowadzaj komunikatu nr 302
	
	org	h'00'				;adres poczatku programu
	goto	start			;skok do poczatku programu

	
start		;pocztek programu


	;konfiguracja procesora

	;wylacz komparator
	movlw	.7				;wylacz komparator
	movwf	cmcon			;

	movlw	b'00100000'		;inicjalizacja linii portu
	movwf	gpio

	bsf	status, rp0			;bank 1

	;fabryczna kalibracja oscylatora

	call	h'3ff'			;fabryczna wartosc kalibracyjna -> w
	movwf	osccal			;w -> osccal

	movlw	b'00011111'		;gp0, gp1, gp2, gp4 - wejscia, gp5 - wyjscie
	movwf	trisio

	;konfiguracja przetwornika ac

	movlw	b'00111111'		;gp0, gp1, gp2, gp4 wejscia analogowe,wewn gen rc
	movwf	ansel 				;

	bcf		status, rp0		;bank 0

	;wybor justyfikacji, vref i zalaczenie przetw. a/c
	clrf	adcon0			;zeruj adcon0
	bsf		adcon0, adfm	;justyfikacja w prawo
;	bsf		adcon0, vcfg	;zewn.nap odniesienia dla przetwornila a/c na GP1 
	bsf		adcon0, adon	;zalacz przetwornik a/c

	;konfiguracja licznika 1
	movlw	b'00110000'		;preskaler = 8, wewn. zegar 
	movwf	t1con

	clrf	tmr1h			;zeruj rejestr tmr1h
	clrf	tmr1l			;zeruj rejestr tmr1l

	clrf	intcon			;zablokuj przerwania
	clrf	pir1			;zeruj flagi - t1if

	bsf		t1con, tmr1on	;zalacz timer 1



;	call	kalibracja		;wywolanie procedury kalibracji
;przy uzywaniu procedury kalibracji nalezy zamienic stala "zero" wartosciami odczytanymi z pamieci
;podobnie moze ulec zmianie algorytm do przeliczania surowego wyniku na stopnie C


;===============================================================
loop		;petla glowna

	;odmierzanie czasu ok. 0.5 sek
	btfss	pir1, t1if		;sprawdz flage t1if
	goto	loop			;czekaj na przepelnienie timera 1
							;nastapilo przepelnienie timera 1 ( co 0.524 sek)
	bcf		pir1, t1if		;zeruj flage przepelnienia

	movlw	.0				;ustaw kanal 0 - GP0
	movwf	nr_kan

	call 	ac				;wykonaj pomiar
	call	skal_term		;skaluje temperature w stopniach
	call	wysw_temp		;wyswietla temperature na terminalu


	movlw	.1				;ustaw kanal 2 - GP2
	movwf	nr_kan

	call 	ac				;wykonaj pomiar
	call	skal_term		;skaluje temperature w stopniach
	call	wysw_temp		;wyswietla temperature na terminalu


	movlw	.3				;ustaw kanal 3 - GP4
	movwf	nr_kan

	call 	ac				;wykonaj pomiar
	call	skal_term		;skaluje temperature w stopniach
	call	wysw_temp		;wyswietla temperature na terminalu


	;kolejny pomiar wyswietlany jest w nowej linii

	movlw	h'0A'			;nowa linia
	call	tx_out

	movlw	h'0D'			;przejdz na poczatek linii
	call	tx_out


	;inne procedury w petli wykonywane co 0.5 sek
	;...

	
	goto 	loop			;powrot do petli
			

;================================================================================
;================================================================================
;procedury
;================================================================================
;================================================================================

;================================================================================
ac		;procedura konwersji ac

	;wybor kanalu 
	bcf		adcon0, chs0
	btfsc	nr_kan, 0		;ustal bit chs0
	bsf		adcon0, chs0
	
	bcf		adcon0, chs1
	btfsc	nr_kan, 1		;ustal bit chs1
	bsf		adcon0, chs1
	
	call	del_acq			;odczekaj czas akwizycji
	
	bsf		adcon0, go		;start konwersji 
	btfsc	adcon0, go		;czy koniec konwersji ?
	goto	$-1				;czekaj na koniec konwersji
	
	return					;wynik konw. w adresh i adresl
	
del_acq		;opoznienie na czas akwizycji 10 us
			;10 cykli maszynowych razem z wywolaniem i powrotem
	
	goto $+1				;opoznienie 2 cykle maszynowe
	goto $+1				;opoznienie 2 cykle maszynowe
	goto $+1				;opoznienie 2 cykle maszynowe
	
	return


;================================================================================
skal_term	;skalowanie termometru

;skalowanie termometru TC1047A - obliczamy wynik w stopniach C i znak temperatury
;rozdzielczosc - 0.25 stopnia


;Vref = 2.500 V (2500 mV), czulosc = 10 mv/stopien
;U(T)= 10*T + 500 (w mV), T - temp w stopniach Celsiusza
;x - wynik pomiaru, rozdzielczosc 10 bitow, U=x*2500/1024 (w mV)
;rownanie pomiaru 10*T + 500 = x*2500/1024
;T = (x*2500/1024-500)/10 = x*250/1024-50
;realizacja: x*250/1024 = (x*(256 - 4 - 2)/256)/4 = (x - x/64 - x/128)/4 
;czyli: x*250/1024 = (x - x/64 - x/128)/4
;wyniki w rejestrach xth,  xtl i xtd (ulamki stopnia)

	cblock

	xth, xtl, xtd, xtd1		;rejestry pomocnicze
	znak					;rejestr znaku

	endc

;skal_term	;skalowanie termometru

	clrf	znak			;zeruj flage minus

	;odczyt danych z przetwornika a/c

	movf	adresh, w
	movwf	xth				;adresh -> xth

	bsf		status, rp0		;bank1
	movf	adresl, w
	bcf		status, rp0		;bank0
	movwf	xtl				;adresl -> xtl

	;sprawdzamy znak temperatury (>=0 dla x >= zero )

	movf	xth, f
	btfss	status, z		;czy xth = 0
	goto 	skaluj			;xth > 0
	movlw	zero			;zero termometru
	subwf	xtl, w			;xtl - zero -> w
	btfsc	status, c		;sprawdz pozyczke
	goto	skaluj			;nie bylo, temp. nieujemna
	bsf		minus			;temperatura ujemna, zaznacz znak minus

skaluj		;skalowanie wyniku  T = ((x*250/1024) - 50)

	rlf		xtl, f
	rlf		xth, f			;xth = x/128
	rlf		xtl, f
	rlf		xth, w			;w = x/64

	addwf	xth, w			;w = x/64 + x/128

	bsf		status, rp0		;bank1
	subwf	adresl, w		;w = adresl - (x/64 + x/128), ustala flage c
	bcf		status, rp0		;bank0

	movwf	xtl				;adresl - (x/64 + x/128) wpisz do xtl

	movf	adresh, w
	movwf	xth				;adresh -> xth
	btfss	status, c		;sprawdz pozyczke
	decf	xth, f			;byla, odejmij 1 od xth

	;podziel przez 4
	
	clrf	xtd				;zeruj rejestr na czesci dziesietne stopnia

	bcf		status, c
	rrf		xth, f				
	rrf		xtl, f			;/2	
	rrf		xtd, f			;zapamietaj 1/4 stop

	bcf		status, c
	rrf		xth, f				
	rrf		xtl, f			;/2		
	rrf		xtd, f			;zapamietaj 1/2 stop

	btfsc	minus			;sprawdz znak wyniku
	goto	ujm				;ujemny
	;dodatni
	movlw	.50
	subwf	xtl, f			;xtl-50 -> xtl
	btfss	status, c		;sprawdz pozyczke
	decf	xth, f			;byla, odejmij 1 od xth
	goto	dzies			;oblicz czesci dziesietne

ujm		;wynik ujemny, xtl = 49 - xtl
	movf	xtl, w			;xtl -> w
	sublw	.49				;49 - xtl -> w
	movwf	xtl				;w -> xtl

	;skoryguj poprawke i xtl
	movlw	b'11000000'		
	xorwf	xtd, w			;zaneguj dwa bity poprawki
	addlw	b'01000000'		;dodaj LSB
	movwf	xtd
	btfsc	status, c		;sprawdz przeniesienie
	incf	xtl, f			;skoryguj xtl


dzies		;oblicza czesci dziesietne - tylko dwa bity, mnozymy przez 25 (25=16+8+1)

	;*16
	bcf		status, c
	rrf		xtd, f 						
	bcf		status, c
	rrf		xtd, f

	swapf	xtd, w			;xtd/16 -> w
	movwf	xtd1			;zapamietaj w xtd1

	;*8
	bcf		status, c
	rrf		xtd, w

	;16+8 = 24,  wynik *24
	addwf	xtd, f 
	
	;24 +1 =25, wynik *25
	movf	xtd1, w
	addwf	xtd, f	


	return


;================================================================================
wysw_temp	;wyswietla temp w stopniach,
			;wynik w xth, xtl, xtd, numer kanalu w nr_kan	

		;wyswietla nrumer kanalu spacje wynik dwie spacje

	movlw	A'T'			;kod litery T
	call	tx_out			;wyslij

	movf	nr_kan, w
	btfsc	status, z		;sprawdz czy nr kanalu = 0
	addlw	.1				;tak, dodaj 1, numeracja kanalow temp 1, 2, 3
	andlw	b'00000011'		;ograniczenie 0, 1, 2, 3
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij

	movlw	.32				;kod spacji
	call	tx_out			;wyslij


	;wpisz wynik do rej. hbyte i lbyte
	movf	xth, w
	movwf	hbyte
	movf	xtl, w
	movwf	lbyte

	call	bin2bcd			;zamien na bcd

;	;setki tys.
;	swapf	r2, w			;zamien polbajty r2 i wpisz do w
;	andlw	h'0F'			;tylko mlodszy bajt
;	addlw	.48				;zamiana na ASCII
;	call	tx_out			;wyslij
;	;dzies. tys.
;	movf	r2, w			;wpisz r2 do w
;	andlw	h'0F'			;tylko mlodszy bajt
;	addlw	.48				;zamiana na ASCII
;	call	tx_out			;wyslij

;	;tysiace
;	swapf	r1, w			;zamien polbajty r1 i wpisz do w
;	andlw	h'0F'			;tylko mlodszy bajt
;	addlw	.48				;zamiana na ASCII
;	call	tx_out			;wyslij

	;setki
	movf	r1, w			;wpisz r1 do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	btfsc	minus			;sprawdz znak 
	movlw	.45				;ujemny, wpisz minus
	call	tx_out			;wyslij
	;dziesiatki
	swapf	r0, w			;zamien polbajty r0 i wpisz do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij
	;jednosci
	movf	r0, w			;wpisz r0 do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij

	;przecinek dziesietny
	movlw	.44				;kod ,
	call	tx_out			;wyslij

	;wpisz ulamki stopnia
	clrf	hbyte			;straszy bajt = 0
	movf	xtd, w
	movwf	lbyte

	call	bin2bcd			;zamien na bcd

	;dziesiate czesci stopnia
	swapf	r0, w			;zamien polbajty r0 i wpisz do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij
	;setne czesci stopnia
	movf	r0, w			;wpisz r0 do w
	andlw	h'0F'			;tylko mlodszy bajt
	addlw	.48				;zamiana na ASCII
	call	tx_out			;wyslij

	movlw	.39				;kod ' (stopnie)
	call	tx_out			;wyslij
	movlw	A'C'			;kod litery C
	call	tx_out			;wyslij

	;dwie spacje odstepu
	movlw	.32				;kod spacji
	call	tx_out			;wyslij
	movlw	.32				;kod spacji
	call	tx_out			;wyslij


	return


;================================================================================

tx_out		;wysyla bit startu, 8 bitow danych i bit stopu

	movwf	tx_reg			;wpisz dane do rejestru tx_reg

	movlw	.9
	movwf	tx_cnt			;licznik bitow (1+8)

	bcf	status, c			;wpisz bit startu

tx_loop		;petla do nadwania bitow
	
	btfss	status, c		;sprawdz c
	bcf		out				;c=0, zeruj out
	btfsc	status, c		;sprawdz c
	bsf		out				;c=1, ustaw out

	call	del_ntb			;opoznienie na czas 1 bitu
	
	rrf		tx_reg, f		;przesun do c kolejny bit

	decfsz	tx_cnt, f		;sprawdz czy nadano wszystkie bity
	goto	tx_loop			;nie, nadawaj kolejny bit

	bsf		status, c		;wpisz bit stopu

	btfss	status, c		;sprawdz c
	bcf		out				;c=0, zeruj out
	btfsc	status, c		;sprawdz c
	bsf		out				;c=1, ustaw out

	goto	$+1				;opoznienie wyrownawcze
	goto	$+1				;opoznienie wyrownawcze
	call	del_ntb			;opoznienie na czas 1 bitu, nadaje pierwszy bit stopu

;	drugi bit stopu
;	goto	$+1				;opoznienie wyrownawcze
;	goto	$+1				;opoznienie wyrownawcze
;	goto	$+1				;opoznienie wyrownawcze
;	call	del_ntb			;opoznienie na czas 1 bitu, nadaje drugi bit stopu

	return

del_ntb		;odmierza opoznienie na czas trwania 1 bitu

	movlw	ntb				;stala do odmierzania czasu 1 bitu	
	movwf	del_reg			;wpisz wartosc poczatkowa do licznika 

	decfsz	del_reg, f		;sprawdz czy del_reg =0 (petla do odliczania opoznienia)
	goto	$-1				;nie, czekaj dalej
	
	return


;================================================================================
;konwersja bin na bcd, wersja 16 bitowa

	;deklaracje rejestrow
	 cblock  
     	r0, r1, r2		    ;liczba dziesietna spakowana (6 cyfr)
     	lbyte, hbyte		;liczba binarna (2 bajty)
	 	cb0, cb1			;liczniki pomocnicze
     endc    

	;r0	dziesiatki 	jednosci
	;r1	tysiace		setki
	;r2	setki tys.	dzies. tys

	;lbyte	mlodszy bajt
	;hbyte	starszy bajt
	
bin2bcd movlw   .16			;liczba bitow
        movwf   cb0			;wpisz do licznika bitow
        clrf    r0
        clrf    r1
        clrf    r2

bin2bcd1 
		movlw   r0
        movwf   fsr			;wskaznik na r0
		movlw	.3			;liczba bajtow wyniku
		movwf	cb1			;wpisz do licznika

bin2bcd2 		;korekta dziesietna
		movlw   h'33'		
        addwf   indf, f		;dodaj h'33'
        btfsc   indf, 3		;sprawdz czy dolny polbajt > 7
		andlw	h'F0'		;tak, zeruj dolny polbajt w
        btfsc   indf, 7		;sprawdz czy gorny polbajt > 7
		andlw	h'0F'		;tak, zeruj gorny polbajt w
        subwf   indf, f		;wynik =< 7, odejmij poprawke
        incf    fsr, f		;nastepny bajt danych
		decfsz	cb1, f			;nastepne bajty wyniku
        goto    bin2bcd2

		;przesun kolejny bit
		rlf     lbyte, f
        rlf     hbyte, f
        rlf     r0,f
        rlf     r1,f
        rlf     r2,f

        decfsz  cb0, f		;czy wszystkie bity
        goto    bin2bcd1

        retlw   0

;		end					;koniec programu bez kalibracji


;================================================================================
kalibracja		;kalibracja termometru w temp 0 stopni C (wszystkie 3 kanaly)
				;wejscie do kalibracji - nacisniecie przycisku przez ponad 1 sek

;przy uzywaniu procedury kalibracji nalezy zamienic stala "zero" wartosciami odczytanymi z pamieci
;podobnie moze ulec zmianie algorytm do przeliczania surowego wyniku na stopnie C

;nalezy zmienic konfiguracje procesora 

;	;konfiguracja procesora, fclk = 4 MHz z wykorzystaniem przycisku kal
;	__config _INTRC_OSC_NOCLKOUT &_MCLRE_OFF &_WDT_OFF


;mapa wewnetrznej pamieci EEPROM
;zero dla kanalu 0 adresy: 0 - MSB, 1- LSB
;zero dla kanalu 1 adresy: 2 - MSB, 3- LSB	- w tej wersji nieuzywane
;zero dla kanalu 2 adresy: 4 - MSB, 5- LSB
;zero dla kanalu 3 adresy: 6 - MSB, 7- LSB

;znacznik kalibracji - adres: 8 (0 - brak kalibracji , 1 - byla kalibracja) 


#define kal	gpio, 3		;przycisk kal podlaczony do gp3
#define n_usr .16		;liczba usrednien  musi byc 2^k

	cblock

	zer_h,	zer_l		;rejestry do usredniania
	usr_cnt				;licznik usrednien

	endc

		
	btfsc	kal			;sprawdz przycisk
	return				;zwolniony, wyjscie z kalibracji
	
		;nacisniety, odmierza czas 0.5 sek

	clrf	tmr1h
	clrf	tmr1l
	bcf		pir1, t1if	;zeruj flage przepelnienia


kal_wait1		;czekaj 0.5 sek i sprawdzaj czy nacisniety przycisk

	btfsc	kal			;sprawdz przycisk
	return				;zwolniony, wyjscie z kalibracji

	btfss	pir1, t1if	;sprawdz flage t1if
	goto	kal_wait1	;czekaj dalej i sprawdzaj przycisk

	;nastapilo przepelnienie timera 1 ( co 0.524 sek)
	bcf		pir1, t1if	;zeruj flage przepelnienia

kal_wait2		;czekaj 0.5 sek i sprawdzaj czy nacisniety przycisk

	btfsc	kal			;sprawdz przycisk
	return				;zwolniony, wyjscie z kalibracji

	btfss	pir1, t1if	;sprawdz flage t1if
	goto	kal_wait2	;czekaj dalej i sprawdzaj przycisk


	;wejscie do kalibracji

	movlw	.0		;kanal 0
	movwf	nr_kan		


kal_loop		;usrednianie kolejnych kanalow

	;omija kanal 1 - vref
	movlw	.1
	xorwf	nr_kan,w	;sprawdz czy nr_kan = 1
	skpnz				;nie, usredniaj
	goto	nast_kan	;tak, przejdz do nast kanalu	

	movlw	n_usr		;liczba usrednien  musi byc 2^k
	movwf	usr_cnt		;wpisz do licznika usrednien

	clrf	zer_h
	clrf	zer_l

usr_loop				;usrednianie wynikow z pojedynczego kanalu

	call	ac			;wykonaj polmiar

	bsf		status, rp0	;bank 1
	movf	adresl, w
	bcf		status, rp0	;bank 0

	;usrednianie
	addwf	zer_l, f
	movf	adresh, w
	btfsc	status, c
	addlw	.1
	addwf	zer_h, f

	decfsz	usr_cnt, f
	goto	usr_loop


;normalizacja
	
	movlw	n_usr/2		;liczba usrednien  musi byc 2^k
	movwf	usr_cnt

norm_loop	

	bcf	status, c
	rrf	zer_h, f
	rrf	zer_l, f		;/2

	bcf	status, c
	rrf	usr_cnt, f		;usr_cnt/2

	btfss	status, c
	goto	norm_loop


;zapis do eepromu - mapa pamieci 
;MSB - adres 2*nr_kan
;LSB - adres 2*nr_kan+1

;zero dla kanalu 0 adresy: 0 - MSB, 1- LSB
;zero dla kanalu 1 adresy: 2 - MSB, 3- LSB
;zero dla kanalu 2 adresy: 4 - MSB, 5- LSB
;zero dla kanalu 3 adresy: 6 - MSB, 7- LSB

;znacznik kalibracji - adres: 8 (0 - brak kalibracji , 1 - byla kalibracja) 

	;zapis MSB
	bcf		status, c		;c=0
	rlf		nr_kan, w		;2*nr_kan	
	bsf		status, rp0		;bank 1
	movwf	eeadr
	bcf		status, rp0		;bank 0
	movf	zer_h, w		;MSB -> w
	call 	eewrite			;zapis MSB
	
	;zapis LSB
	bsf		status, c		;c=1
	rlf		nr_kan, w		;2*nr_kan+1
	bsf		status, rp0		;bank 1
	movwf	eeadr
	bcf		status, rp0		;bank 0
	movf	zer_l, w		;LSB -> w
	call	eewrite			;zapis LSB

nast_kan	;numer nastepnego kanalu
	incf	nr_kan, f		;zwieksz nr_kan
	movlw	.4
	subwf	nr_kan, w		;nr_kan - 4 -> w
	btfss	status, z		;sprawdz czy wszystkie kanaly
	goto	kal_loop		;nie, kolejny kanal


	;zapis znacznika usredniania
	movlw	.8				;adres wskaznika
	bsf		status, rp0		;bank 1
	movwf	eeadr
	bcf		status, rp0		;bank 0
	movlw	.1				;ustaw znacznik = 1
	call	eewrite			;zapis LSB

	return	;koniec kalibracji


;================================================================================
eewrite	;zapis do pamieci eeprom

	bsf		status,rp0	;bank 1
	movwf	eedata		; w -> eedata
	
	bsf	eecon1,wren		;zezwolenie na zapis
	bcf	intcon,gie		;wylaczenie przerwan

	;sekwencja zapisu
	movlw	h'55'					
	movwf	eecon2						
	movlw	h'AA'						
	movwf	eecon2						
	bsf		eecon1,wr	;rozpoczecie zapisu		
	
	btfsc	eecon1, wr	;czy zakonczony zapis
	goto	$-1			;czekaj

	bcf		eecon1,wren	;wylaczenie zezwolenia na zapis
	bcf		status,rp0	;bank 0

	return


;================================================================================
eeread		;odczyt z pamieci eeprom, adres w rejestrze w
	
	bsf		status, rp0	;bank 1
	movwf	eeadr		;zapis adresu do rejestru eeadr
	bsf		eecon1, rd	;inicjalizacja odczytu
	movf	eedata, w	;zapisz odczytany bajt do rejestru w
	bcf		status, rp0	;bank 0
	
	return


;================================================================================


	
	end

