;i2c_test - test magistrali i2c
;zapis i odczyt danych do pamieci 24c02, 24c04
;po zalaczeniu procesora zapisuje do komorki nr 16 wartosc 55h i odczytuje ja
;jezeli odczytana wartosc = 55h zapala diode podlaczona do gp2 na 65 ms

;podlaczenia pamieci do procesora
;sda	gpio, 4
;scl	gpio, 5
;adres ustawiony na pamieci = 0 (a0=0, a1=0, a2=0)
;adres pamieci 24c02 = 1010
;linia wp(pin7 ukladu 24c02) zwarta do masy (zapis odblokowany)


;========================================================================
;i2c_def		;definicje rejestrow i oznaczen

	cblock	h'20'

	i2c_reg			;rejestr na dane do procedur i2c	
	i2c_state		;rejestr stanu do procedur i2c
	licznik			;pomocniczy licznik do procedur i2c

	endc

#define		sda	gpio, 4
#define		scl	gpio, 5

#define		led	gpio, 2

#define		ack	i2c_state, 0

#define		read	iorlw	b'00000001'		;ustawia bit r/w
#define		write	andlw	b'11111110'		;zeruje  bit r/w

#define		bank0	bcf	status, rp0
#define		bank1	bsf	status, rp0

#define		ze	status, z					;flaga z
#define		ca	status, c					;flaga c

#define		adr_pam		b'10100000'			;adres pamieci 24c04
#define		adr_kom		.16					;adres komorki w pamieci 24c04
#define		test		h'55'				;dana testowa = h'55'


	;deklaracja typu procesora i formatu liczb
	list p=12F675			;okreslenie typu procesora
	#include p12f675.inc	;dolaczenie pliku z nazwami

	;konfiguracja procesora
	__config _INTRC_OSC_NOCLKOUT &_MCLRE_ON &_WDT_OFF


;========================================================================
	org		0

	;wylacz komparator
	movlw	.7
	movwf	cmcon

	;wylacz A/C
	bsf		status, rp0		;bank1
	clrf	ansel
	bcf		status, rp0		;bank0

	movlw	b'00111000'
	movwf	gpio			;inicjalizacja portow

	;konfiguracja linii i/o
	bsf		status, rp0		;bank1
	movlw	b'00111000'
	movwf	trisio			;GP3, gp4, gp5 - wejscia, gp0, gp1, gp2 - wyjscia
	bcf		status, rp0		;bank0

	clrf	pir1
	clrf	intcon

	call	del_64ms		;czekaj 64 ms po zalaczeniu zasilania
	
;========================================================================
;program glowny
;========================================================================


;========================================================================
;zapis wartosci "test" do wybranej komorki pamieci 

	call	i2c_start

	movlw	adr_pam			;adres pamieci -> w
	write					;ustaw zapis (r/w=0)
	call	i2c_send		;wyslij rozkaz
	
	btfss	ack				;sprawdz potwierdzenie
	goto	wait			;nie bylo, skok na koniec 

	movlw	adr_kom			;adres komorki -> w 
	call	i2c_send		;wyslij adres
	
	movlw	test			;dana do zapisu, test -> w 
	call	i2c_send		;wyslij dane
	
	call	i2c_stop	
	
	call del_10ms			;opoznienie na czas zapisu, max 10 ms

	
;========================================================================
;odczyt wybranej komorki z pamieci 
	
	call	i2c_start
	
	movlw	adr_pam			;adres pamieci -> w
	write					;ustaw zapis (r/w=0)
	call	i2c_send		;wyslij rozkaz

	movlw	adr_kom			;adres komorki -> w 
	call	i2c_send		;wyslij adres
	
	call	i2c_start	
	
	movlw	adr_pam			;adres pamieci -> w
	read					;ustaw odczyt (r/w=1)
	call	i2c_send		;wyslij rozkaz
	
	bcf		ack				;nie wysylaj potwierdzenia	
	call	i2c_read		;odczytaj zawartosc komorki

	call	i2c_stop
	

;========================================================================	
;sprawdzenie czy odczytana dana = stala "test"	
	movf	i2c_reg, w		;i2c_reg -> w
	xorlw	test			;porownaj zestala test
	btfss	status, z		;sprawdz flage z
	goto	wait			;0, nie zgadza sie
							;1, zgadza sie, zapal diode
						
	bsf		led				;zapal led					
	call	del_64ms		;czekaj 64 ms					
	bcf		led				;zgas led


wait			;czekaj tu na wyzerowanie procesora

	goto	$	;koniec programu glownego
	

;========================================================================


;========================================================================
;procedury i2c
;========================================================================


;========================================================================
;listing 7.18 procedura inicjalizacji rejestrow portu dla linii sda i scl
;========================================================================

i2c_port_ini	;inicjalizacja rejestrow portow
	
	bank1
	bsf		sda				;linia sda - wejscie
	bsf		scl				;linia scl - wejscie
	bank0
	
	return


;========================================================================
;listing 7.23 procedura start - inicjalizacja transmisji
;========================================================================

i2c_start		;wysyla sekwencje start

	bank1
	bcf	sda					;sda wyjscie
	bcf	scl					;scl wyjscie
	bank0
	
	bsf		sda				;1 -> sda
	bsf		scl				;1 -> sdcl
	call	del6			;czekaj 6 us
	
	bcf		sda				;0 -> sda
	call	del6			;czekaj 6 us
	bcf		scl				;0 -> scl

	return					;zostaje sda=0, scl=0


;========================================================================
;listing 7.24 procedura stop - zakonczenie transmisji
;========================================================================

i2c_stop		;wysyla sekwencje stop

	bank1
	bcf	sda				;sda wyjscie
	bcf	scl				;scl wyjscie
	bank0
		
	bcf		sda			;0 -> sda
	goto	$+1			;opoznienie 2us
	bsf		scl			;1 -> sdcl
	call	del6		;czekaj 6 us
	
	bsf		sda			;1 -> sda
	call	del6		;czekaj 6 us
	
	return				;zostaje sda=1, scl=1


;========================================================================
;listing 7.25 procedura wyslania adresu lub danych
;========================================================================

i2c_send			;wysyla adres lub dane z rejestru w
		
	bank0
	movwf	i2c_reg		;wpisz dane do rejestru i2c_reg                                                                                                                                                                                                                                                                                                                                                                    
	
	movlw	.8			;liczba bitow
	movwf	licznik
	
send_bit	;wysyla najstarszy bit z rejestru i2c_reg

	rlf	i2c_reg, f		;przesun bit 7 do c
	bcf		sda			;przygotuj 0 na sda
	bank1
	btfss	ca			;sprawdz c
	bcf		sda			;c=0, 0->sda
	btfsc	ca			;sprawdz c
	bsf		sda			;c=1, 1->sda
	call 	del6		;opoznienie 6 us
	bsf		scl			;wpisz 1 na scl                                                                                                                                                                                                                                                        
	bank0
	btfss	scl			;sprawdz scl
	goto	$-1			;scl=0, czekaj
	goto	$+1			;opoznienie 2us
	bank1
	bcf		scl			;wpisz 0 na scl
	bank0

	decfsz 	licznik, f	;sprawdza liczbe wyslanych bitow
	goto	send_bit	;wyslij kolejny bit
	
	goto	$+1			;opoznienie 2us
	goto	$+1			;opoznienie 2us
	
			;sprawdza ack
	bank1
	bsf		sda			;przygotowanie linii sda do odczytu
	bsf		scl			;wpisz 1 na scl
	bank0
	
	btfss	scl			;sprawdz scl
	goto	$-1			;scl=0, czekaj
	goto	$+1			;opoznienie	
	
	bcf		ack			;zeruj flage ack
	btfss	sda			;czy jest potwierdzenie (sda=0)
	bsf		ack			;jest potwierdzenie, ustaw flage ack  
	call 	del6		;opoznienie 6 us
	                                                                                                                                   
	bank1
	bcf		scl			;wpisz 0 na scl
	bank0
	bcf		scl			;wpisz 0 na scl
	
	return				;zostawia 0 na scl i 1 na sda


;========================================================================
;listing 7.26 procedura odbioru danych
;========================================================================

i2c_read		;odbiera dane do rejestru i2c_reg
				;ack=1 wysyla potwierdzenie
				;ack=0 nie wysyla potwierdzenia
	
	bank1
	bsf		sda			;przygotowanie linii sda do odczytu
	bank0
	
	bcf		scl			;0-> scl
	
	movlw	.8			;liczba bitow
	movwf	licznik
	
read_bit		;odbiera 1 bit 

	bank1
	bsf		scl			;wpisz 1 na scl
	bank0
	
	btfss	scl			;sprawdz scl
	goto	$-1			;scl=0, czekaj
	
	bsf		ca			;wpisz 1 do c
	btfss	sda			;sprawdz czy sda = 1
	bcf		ca			;sda = 0, wpisz 0 do c
	rlf		i2c_reg, f	;wpisz c do rejestru i2c_reg
	
	bank1
	bcf		scl			;wpisz 0 na scl
	bank0
	bcf		scl			;wpisz 0 na scl
	
	decfsz 	licznik, f	;sprawdza liczbe odebranych bitow
	goto	read_bit	;odbierz kolejny bit
	
	goto	$+1			;opoznienie

	bsf		sda
	btfsc	ack			;sprawdz ack
	bcf		sda			;ustaw potwierdzenie
	
	bank1
	bcf		sda			;wyslij porwierdzenie  (0 -> sda)
	bank0
	
	goto	$+1			;opoznienie
	
	bsf		scl			;wpisz 1 na scl
	btfss	scl			;sprawdz scl
	goto	$-1			;scl=0, czekaj
	
	goto	$+1			;opoznienie
;	goto	$+1			;opoznienie
		
	bcf		scl			;wpisz 0 na scl
	goto	$+1			;opoznienie
	
	bank1

	bsf		sda			;zwolnienie linii sda 
	bank0
	
	return				;zostawia 1 na sda , 0 na scl


;========================================================================
;opoznienie 6us do procedur i2c
del6			;czekaj 6 us razem z wywolaniem i powrotem (wewn. zegar 4 MHz)
	goto $+1		;opoznienie 2us
	
	return


;========================================================================	
del_10ms	;opoznienie  10ms (oczekiwanie na zakonczenie zapisu)
	
	clrf	t1con			;konfiguracja timera 1, presk. =1, wewn. zegar
	clrf	tmr1l			;zeruj tmr1l
	movlw	.220
	movwf	tmr1h			;220 -> tmrh
	bcf		pir1, t1if		;zeruj  t1if
	bsf		t1con, tmr1on	;zalacz tmr1

	btfss	pir1, t1if		;sprawdz t1if
	goto	$-1				;czekajna ustawienie flagi

	bcf		t1con, tmr1on	;wylacz tmr1
	bcf		pir1, t1if		;zeruj  t1if

	return


;========================================================================
del_64ms	;opoznienie 64 ms
	
	clrf	t1con			;konfiguracja timera 1, presk. =1, wewn. zegar
	clrf	tmr1l			;zeruj tmr1l
	clrf	tmr1h			;zeruj tmr1h
	bcf		pir1, t1if		;zeruj  t1if
	bsf		t1con, tmr1on	;zalacz tmr1

	btfss	pir1, t1if		;sprawdz t1if
	goto	$-1				;czekajna ustawienie flagi

	bcf		t1con, tmr1on	;wylacz tmr1
	bcf		pir1, t1if		;zeruj  t1if

	return


;========================================================================

	end
	
	
